Package: pqrBayes
Type: Package
Title: Bayesian Penalized Quantile Regression
Version: 1.2.2
Date: 2026-04-28
Authors@R: c( 
              person("Kun", "Fan", role = "aut"),
              person("Cen", "Wu", role = c("aut", "cre"),email = "wucen@ksu.edu"),
              person("Jie", "Ren", role = "aut"),
              person("Xiaoxi", "Li", role = "aut"),
              person("Fei", "Zhou", role = "aut"))
Description: Bayesian regularized quantile regression utilizing two major classes of shrinkage priors 
    (the spike-and-slab priors and the horseshoe family of priors) leads to efficient Bayesian 
    shrinkage estimation, variable selection and valid statistical inference. In this package, 
    we have implemented robust Bayesian variable selection with spike-and-slab priors under 
    high-dimensional linear regression models (Fan et al. (2024) <doi:10.3390/e26090794> and  
    Ren et al. (2023) <doi:10.1111/biom.13670>), and regularized quantile varying
    coefficient models (Zhou et al.(2023) <doi:10.1016/j.csda.2023.107808>). In particular, 
    valid robust Bayesian inferences under both models in the presence of heavy-tailed errors
    can be validated on finite samples. Additional models with spike-and-slab priors include 
    robust Bayesian group LASSO and robust binary Bayesian LASSO (Fan and Wu (2025) 
    <doi:10.1002/sta4.70078>). Besides, robust sparse Bayesian regression with the horseshoe
    family of (horseshoe, horseshoe+ and regularized horseshoe) priors has also been implemented
    and yielded valid inference results under heavy-tailed model errors (Fan et al.(2026) 
    <doi:10.1016/j.csda.2026.108358>). The Markov chain Monte Carlo (MCMC) algorithms of 
    the proposed and alternative models are implemented in C++. 
Depends: R (>= 3.5.0)
License: GPL-2
Encoding: UTF-8
URL: https://github.com/cenwu/pqrBayes
BugReports: https://github.com/cenwu/pqrBayes/issues
LazyData: true
Imports: Rcpp,glmnet,splines, stats
LinkingTo: Rcpp, RcppArmadillo
RoxygenNote: 7.3.3
NeedsCompilation: yes
Repository: CRAN
Packaged: 2026-04-30 03:43:43 UTC; cenwu
Author: Kun Fan [aut],
  Cen Wu [aut, cre],
  Jie Ren [aut],
  Xiaoxi Li [aut],
  Fei Zhou [aut]
Maintainer: Cen Wu <wucen@ksu.edu>
Date/Publication: 2026-04-30 07:10:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-30 11:01:27 UTC; unix
Archs: pqrBayes.so.dSYM

Package: pqrBayes
Type: Package
Title: Bayesian Penalized Quantile Regression
Version: 1.2.2
Date: 2026-04-28
Authors@R: c( 
              person("Kun", "Fan", role = "aut"),
              person("Cen", "Wu", role = c("aut", "cre"),email = "wucen@ksu.edu"),
              person("Jie", "Ren", role = "aut"),
              person("Xiaoxi", "Li", role = "aut"),
              person("Fei", "Zhou", role = "aut"))
Description: Bayesian regularized quantile regression utilizing two major classes of shrinkage priors 
    (the spike-and-slab priors and the horseshoe family of priors) leads to efficient Bayesian 
    shrinkage estimation, variable selection and valid statistical inference. In this package, 
    we have implemented robust Bayesian variable selection with spike-and-slab priors under 
    high-dimensional linear regression models (Fan et al. (2024) <doi:10.3390/e26090794> and  
    Ren et al. (2023) <doi:10.1111/biom.13670>), and regularized quantile varying
    coefficient models (Zhou et al.(2023) <doi:10.1016/j.csda.2023.107808>). In particular, 
    valid robust Bayesian inferences under both models in the presence of heavy-tailed errors
    can be validated on finite samples. Additional models with spike-and-slab priors include 
    robust Bayesian group LASSO and robust binary Bayesian LASSO (Fan and Wu (2025) 
    <doi:10.1002/sta4.70078>). Besides, robust sparse Bayesian regression with the horseshoe
    family of (horseshoe, horseshoe+ and regularized horseshoe) priors has also been implemented
    and yielded valid inference results under heavy-tailed model errors (Fan et al.(2026) 
    <doi:10.1016/j.csda.2026.108358>). The Markov chain Monte Carlo (MCMC) algorithms of 
    the proposed and alternative models are implemented in C++. 
Depends: R (>= 3.5.0)
License: GPL-2
Encoding: UTF-8
URL: https://github.com/cenwu/pqrBayes
BugReports: https://github.com/cenwu/pqrBayes/issues
LazyData: true
Imports: Rcpp,glmnet,splines, stats
LinkingTo: Rcpp, RcppArmadillo
RoxygenNote: 7.3.3
NeedsCompilation: yes
Repository: CRAN
Packaged: 2026-04-30 03:43:43 UTC; cenwu
Author: Kun Fan [aut],
  Cen Wu [aut, cre],
  Jie Ren [aut],
  Xiaoxi Li [aut],
  Fei Zhou [aut]
Maintainer: Cen Wu <wucen@ksu.edu>
Date/Publication: 2026-04-30 07:10:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-30 11:22:58 UTC; unix
Archs: pqrBayes.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(print,pqrBayes)
S3method(print,pqrBayes.pred)
S3method(print,pqrBayes.select)
export(coverage)
export(estimation.pqrBayes)
export(pqrBayes)
export(pqrBayes.select)
export(predict_pqrBayes)
importFrom(Rcpp,sourceCpp)
useDynLib(pqrBayes, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# pqrBayes

> Bayesian Penalized Quantile Regression

<!-- badges: start -->

[![CRAN](https://www.r-pkg.org/badges/version/pqrBayes)](https://cran.r-project.org/package=pqrBayes)
[![CRAN RStudio mirror
downloads](https://cranlogs.r-pkg.org/badges/grand-total/pqrBayes)](https://www.r-pkg.org:443/pkg/pqrBayes)
[![CRAN RStudio mirror
downloads](https://cranlogs.r-pkg.org/badges/last-month/pqrBayes)](https://www.r-pkg.org:443/pkg/pqrBayes)

<!-- badges: end -->


Bayesian regularized quantile regression utilizing two major classes of shrinkage priors 
    (the spike-and-slab priors and the horseshoe family of priors) leads to efficient Bayesian 
    shrinkage estimation, variable selection and valid statistical inference. In this package, we have implemented robust
    Bayesian variable selection with spike-and-slab priors under high-dimensional
    linear regression models ([Fan et al. (2024)](https://doi.org/10.3390/e26090794) and 
    [Ren et al. (2023)](https://doi.org/10.1111/biom.13670)), and regularized quantile varying
    coefficient models ([Zhou et al.(2023)](https://doi.org/10.1016/j.csda.2023.107808)). In   
    particular, valid robust Bayesian inferences under both models in the presence of heavy-tailed
    errors can be validated on finite samples. Additional models with spike-and-slab priors 
    include robust Bayesian group LASSO and robust binary Bayesian LASSO ([Fan and Wu (2025)](https://doi.org/10.1002/sta4.70078)). 
    Besides, robust sparse Bayesian regression with the horseshoe family of (horseshoe, horseshoe+ and 
    regularized horseshoe) priors has also been implemented and yielded valid inference results under heavy-tailed model errors
    ([Fan et al.(2026)](https://doi.org/10.1016/j.csda.2026.108358)). The Markov Chain Monte Carlo (MCMC) 
    algorithms of the proposed and alternative models are implemented in C++. 
   

## How to install

 - To install from github, run these two lines of code in R

<!-- end list -->

    install.packages("devtools")
    devtools::install_github("cenwu/pqrBayes")

- Released versions of pqrBayes are available on CRAN
    [(link)](https://cran.r-project.org/package=pqrBayes),
    and can be installed within R via

<!-- end list -->

    install.packages("pqrBayes")
    
    
## Example 1 (Robust Bayesian Inference for Sparse Linear Regression with Spike-and-Slab Priors)

#### Data Generation for Sparse Linear Model 

    Data <- function(n,p,quant){
      sig1 = matrix(0,p,p)
      diag(sig1)=1
      for (i in 1: p)
      {
      for (j in 1: p)
      {
      sig1[i,j]=0.5^abs(i-j)
      }
     }
    xx = MASS::mvrnorm(n,rep(0,p),sig1)
    x = cbind(1,xx)
    error0 = rt(n,2)
    error= error0 -quantile(error0,probs = quant) # can also be changed to normal error for non-robust setting
    beta = c(0,1,1.5,2,rep(0,p-3))
    betaa = beta[-1]
    y = x%*%beta+error
    dat = list(y=y, x=xx, beta=betaa)
    return(dat)
    }
#### 95% empirical coverage probabilities for sparse linear regression coefficients

    n=100; p=500; rep=1000;
    quant = 0.5; # focus on median for Bayesian inference
    
    CI_RBLSS = CI_RBL = CI_BLSS = CI_BL= matrix(0,rep,p)

    for (h in 1:rep) {
    dat = Data(n,p,quant)
    y = dat$y
    g = dat$x
    coefficient = dat$beta

    # an intercept not subject to regularization is automatically included by the package
    
    # RBLSS: robust Bayesian LASSO with spike-and-slab priors (Ren et al., Biometrics, 2023)
    fit = pqrBayes(g, y ,e=NULL, d = NULL, quant=quant, iterations=10000, burn.in = NULL ,robust = TRUE, prior = "SS", model = "linear", hyper=NULL,debugging=FALSE)
    coverage = coverage(fit,coefficient,u.grid=NULL, model = "linear")
    
    # RBL: Bayesian quantile LASSO (Li, Xi & Lin, Bayesian Analysis, 2010)
    fit1 = pqrBayes(g, y,e=NULL, d = NULL, quant=quant, iterations=10000, burn.in = NULL, robust = TRUE, prior = "Laplace", model = "linear", hyper=NULL,debugging=FALSE)
    coverage1 = coverage(fit1,coefficient,u.grid=NULL, model = "linear")
    
    # BLSS: Bayesian LASSO with spike-and-slab priors (Ren et al., Biometrics, 2023)
    fit2 = pqrBayes(g, y ,e=NULL, d = NULL, quant=NULL, iterations=10000, burn.in = NULL ,robust = FALSE, prior = "SS", model = "linear", hyper=NULL,debugging=FALSE)
    coverage2 = coverage(fit2,coefficient,u.grid=NULL, model = "linear")
    
    # Bayesian LASSO  (Park and Casella, JASA, 2008)
    fit3 = pqrBayes(g, y,e=NULL, d = NULL, quant=NULL, iterations=10000, burn.in = NULL, robust = FALSE, prior = "Laplace", model = "linear", hyper=NULL,debugging=FALSE)
    coverage3 = coverage(fit3,coefficient,u.grid=NULL, model = "linear")
    
    CI_RBLSS[h,] = coverage
    CI_RBL[h,]    = coverage1
    CI_BLSS[h,]   = coverage2
    CI_BL[h,]     = coverage3
    
    cat("Replicate = ", h, "\n")
    
    }
    # the intercept has not been regularized
    cp_RBLSS =  colMeans(CI_RBLSS)[1:3] # 95% empirical coverage probabilities for coefficients under the robust linear model
    cp_RBL    =  colMeans(CI_RBL)[1:3]
    cp_BLSS   =  colMeans(CI_BLSS)[1:3] # 95% empirical coverage probabilities for coefficients under the non-robust linear model
    cp_BL     =  colMeans(CI_BL)[1:3]
    
#### Bayesian shrinkage estimaton via robust Bayesian LASSO with spike-and-slab priors

    n=100; p=500;
    quant = 0.5; # focus on median for Bayesian estimation
    dat = Data(n,p,quant)
    y = dat$y
    g = dat$x
    coefficient = dat$beta

    # an intercept not subject to regularization is automatically included by the package

    # RBLSS: robust Bayesian LASSO with spike-and-slab priors (Ren et al., Biometrics, 2023)

    fit = pqrBayes(g, y ,e=NULL, d = NULL, quant=quant, iterations=10000, burn.in = NULL ,robust = TRUE, prior = "SS", model = "linear", hyper=NULL,debugging=FALSE)

    fit$coefficients$GS.beta # posterior samples for regression coefficients
    estimation_1 = estimation.pqrBayes(fit,coefficient,model="linear")
    coeff_est_1 = estimation_1$coeff.est    
    mse_1 = estimation_1$error$MSE
    
    # RBL: Bayesian quantile LASSO (Li, Xi & Lin, Bayesian Analysis, 2010)

    fit1 = pqrBayes(g, y,e=NULL, d = NULL, quant=quant, iterations=10000, burn.in = NULL, robust = TRUE, prior = "Laplace", model = "linear", hyper=NULL,debugging=FALSE)

    fit1$coefficients$GS.beta # posterior samples for regression coefficients
    estimation_2 = estimation.pqrBayes(fit1,coefficient,model="linear")
    coeff_est_2 = estimation_2$coeff.est    
    mse_2 = estimation_2$error$MSE
    
    # BLSS: Bayesian LASSO with spike-and-slab priors (Ren et al., Biometrics, 2023)
    
    fit2 = pqrBayes(g, y ,e=NULL, d = NULL, quant=NULL, iterations=10000, burn.in = NULL ,robust = FALSE, prior = "SS", model = "linear", hyper=NULL,debugging=FALSE)
    
    estimation_3 = estimation.pqrBayes(fit2,coefficient,model="linear")
    coeff_est_3 = estimation_3$coeff.est
    mse_3 = estimation_3$error$MSE
    
    # Bayesian LASSO  (Park and Casella, JASA, 2008)
    
    fit3 = pqrBayes(g, y,e=NULL, d = NULL, quant=NULL, iterations=10000, burn.in = NULL, robust = FALSE, prior = "Laplace", model = "linear", hyper=NULL,debugging=FALSE)
    
    estimation_4 = estimation.pqrBayes(fit3,coefficient,model="linear")
    coeff_est_4 = estimation_4$coeff.est
    mse_4 = estimation_4$error$MSE
    
## Example 2 (Robust Bayesian Inference for Sparse Linear Regression with the Horseshoe Family of Priors)

#### Data Generation for Sparse Linear Model 

    Data <- function(n,p,quant){
      sig1 = matrix(0,p,p)
      diag(sig1)=1
      for (i in 1: p)
      {
      for (j in 1: p)
      {
      sig1[i,j]=0.5^abs(i-j)
      }
     }
    xx = MASS::mvrnorm(n,rep(0,p),sig1)
    x = cbind(1,xx)
    error0 = rt(n,2)
    error= error0 -quantile(error0,probs = quant) # can also be changed to normal error for non-robust setting
    beta = c(0,1,1.5,2,rep(0,p-3))
    betaa = beta[-1]
    y = x%*%beta+error
    dat = list(y=y, x=xx, beta=betaa)
    return(dat)
    }
#### 95% empirical coverage probabilities for linear regression coefficients

    n=100; p=500; rep=1000;
    quant = 0.5; # focus on median for Bayesian inference
    
    CI_RBHS = CI_RBHS_plus = CI_RBRHS = CI_BHS= CI_BHS_plus = CI_BRHS =matrix(0,rep,p)

    for (h in 1:rep) {
    dat = Data(n,p,quant)
    y = dat$y
    g = dat$x
    coefficient = dat$beta

    # an intercept not subject to regularization is automatically included by the package
    
    
    # RBHS: robust Bayesian Regression with horseshoe priors
    fit1 = pqrBayes(g, y ,e=NULL, d = NULL, quant=quant, iterations=10000, burn.in = NULL, robust = TRUE, prior = "HS", model = "linear", hyper=NULL,debugging=FALSE)
    coverage1 = coverage(fit1,coefficient,u.grid=NULL, model = "linear")
    
    # RBHS+: robust Bayesian Regression with horseshoe plus priors
    fit2 = pqrBayes(g, y ,e=NULL, d = NULL, quant=quant, iterations=10000, burn.in = NULL, robust = TRUE, prior = "HS+", model = "linear", hyper=NULL,debugging=FALSE)
    coverage2 = coverage(fit2,coefficient,u.grid=NULL, model = "linear")
    
    # RBRHS: robust Bayesian Regression with regularized horseshoe priors
    fit3 = pqrBayes(g, y ,e=NULL, d = NULL, quant=quant, iterations=10000, burn.in = NULL, robust = TRUE, prior = "RHS", model = "linear", hyper=NULL,debugging=FALSE)
    coverage3 = coverage(fit3,coefficient,u.grid=NULL, model = "linear")
    
    # BHS: Bayesian Regression with horseshoe priors
    fit4 = pqrBayes(g, y ,e=NULL, d = NULL, quant=NULL, iterations=10000, burn.in = NULL, robust = FALSE, prior = "HS", model = "linear", hyper=NULL,debugging=FALSE)
    coverage4=coverage(fit4,coefficient,u.grid=NULL, model = "linear")
    
    # BHS+: Bayesian Regression with horseshoe plus priors
    fit5 = pqrBayes(g, y ,e=NULL, d = NULL, quant=NULL, iterations=10000, burn.in = NULL, robust = FALSE, prior = "HS+", model = "linear", hyper=NULL,debugging=FALSE)
    coverage5 = coverage(fit5,coefficient,u.grid=NULL, model = "linear")
    
    # BRHS: Bayesian Regression with regularized horseshoe priors
    fit6 = pqrBayes(g, y ,e=NULL, d = NULL, quant=NULL, iterations=10000, burn.in = NULL, robust = FALSE, prior = "RHS", model = "linear", hyper=NULL,debugging=FALSE)
    coverage6 = coverage(fit6,coefficient,u.grid=NULL, model = "linear")
    
    
    CI_RBHS[h,]   = coverage1
    CI_RBHS_plus[h,]  = coverage2
    CI_RBRHS[h,]   = coverage3
    CI_BHS[h,]    = coverage4
    CI_BHS_plus[h,]   = coverage5
    CI_BRHS[h,]  = coverage6
    
    cat("Replicate = ", h, "\n")
    
    }
    # the intercept has not been regularized
  
    cp_RBHS  =  colMeans(CI_RBHS)[1:3]
    cp_RBHS_plus   =  colMeans(CI_RBHS_plus)[1:3]
    cp_RBRHS    =  colMeans(CI_RBRHS)[1:3]
    cp_BHS    =  colMeans(CI_BHS)[1:3]
    cp_BHS_plus   =  colMeans(CI_BHS_plus)[1:3]
    cp_BRHS   =  colMeans(CI_BRHS)[1:3]

#### robust Bayesian shrinkage with the horseshoe family of priors

    n=100; p=500;
    quant = 0.5; # focus on median for Bayesian estimation
    dat = Data(n,p,quant)
    y = dat$y
    g = dat$x
    coefficient = dat$beta

    # an intercept not subject to regularization is automatically included by the package

    # RBHS: robust Bayesian Regression with horseshoe priors

    fit1 = pqrBayes(g, y ,e=NULL, d = NULL, quant=quant, iterations=10000, burn.in = NULL, robust = TRUE, prior = "HS", model = "linear", hyper=NULL,debugging=FALSE)
    
    fit1$coefficients$GS.beta # posterior samples for regression coefficients
    estimation_1 = estimation.pqrBayes(fit1,coefficient,model="linear")
    coeff_est_1 = estimation_1$coeff.est    
    mse_1 = estimation_1$error$MSE

    # RBHS+: robust Bayesian Regression with horseshoe plus priors

    fit2 = pqrBayes(g, y ,e=NULL, d = NULL, quant=quant, iterations=10000, burn.in = NULL, robust = TRUE, prior = "HS+", model = "linear", hyper=NULL,debugging=FALSE)

    fit2$coefficients$GS.beta # posterior samples for regression coefficients
    estimation_2 = estimation.pqrBayes(fit2,coefficient,model="linear")
    coeff_est_2 = estimation_2$coeff.est    
    mse_2 = estimation_2$error$MSE

    # RBRHS: robust Bayesian Regression with regularized horseshoe priors

    fit3 = pqrBayes(g, y ,e=NULL, d = NULL, quant=quant, iterations=10000, burn.in = NULL, robust = TRUE, prior = "RHS", model = "linear", hyper=NULL,debugging=FALSE)
 
    fit3$coefficients$GS.beta # posterior samples for regression coefficients
    estimation_3 = estimation.pqrBayes(fit3,coefficient,model="linear")
    coeff_est_3 = estimation_3$coeff.est    
    mse_3 = estimation_3$error$MSE
    
    # BHS: Bayesian Regression with horseshoe priors
    fit4 = pqrBayes(g, y ,e=NULL, d = NULL, quant=NULL, iterations=10000, burn.in = NULL, robust = FALSE, prior = "HS", model = "linear", hyper=NULL,debugging=FALSE)
 
    estimation_4 = estimation.pqrBayes(fit4,coefficient,model="linear")
    coeff_est_4 = estimation_4$coeff.est
    mse_4 = estimation_4$error$MSE
    
    # BHS+: Bayesian Regression with horseshoe plus priors
    fit5 = pqrBayes(g, y ,e=NULL, d = NULL, quant=NULL, iterations=10000, burn.in = NULL, robust = FALSE, prior = "HS+", model = "linear", hyper=NULL,debugging=FALSE)
 
    estimation_5 = estimation.pqrBayes(fit5,coefficient,model="linear")
    coeff_est_5 = estimation_5$coeff.est
    mse_5 = estimation_5$error$MSE
    
    # BRHS: Bayesian Regression with regularized horseshoe priors
    fit6 = pqrBayes(g, y ,e=NULL, d = NULL, quant=NULL, iterations=10000, burn.in = NULL, robust = FALSE, prior = "RHS", model = "linear", hyper=NULL,debugging=FALSE)

    estimation_6 = estimation.pqrBayes(fit6,coefficient,model="linear")
    coeff_est_6 = estimation_6$coeff.est
    mse_6 = estimation_6$error$MSE
    

## Example 3 (Robust Bayesian Inference for Sparse Varying Coefficients)
#### Data Generation for the Varying Coefficient Model 

    Data <- function(n,p,quant){
      sig1 = matrix(0,p,p)
      diag(sig1)=1
      for (i in 1: p)
      {
      for (j in 1: p)
      {
      sig1[i,j]=0.5^abs(i-j)
      }
     }
    x = MASS::mvrnorm(n,rep(0,p),sig1)
    x = cbind(1,x)
    error0 = rt(n,2)
    error= error0 -quantile(error0,probs = quant)
    u = runif(n,0.01,0.99)
    gamma0 = 2+2*sin(u*2*pi)
    gamma2 = -6*u*(1-u)
    gamma1 = 2*exp(2*u-1)
    gamma3= -4*u^3
    y = gamma1*x[,2] + gamma2*x[,3]  + gamma3*x[,4] + gamma0 + error
    dat = list(y=y, u=u, x=x, gamma=cbind(gamma0,gamma1,gamma2,gamma3))
    return(dat)
    }
#### 95% empirical coverage probabilities for sparse varying coefficients

    n=200; p=100; # the actual dimension after basis expansion is 505
    rep=200;
    quant = 0.5; # focus on median for Bayesian inference

    CI_BQRVCSS = CI_BQRVC = CI_BVCSS = CI_BVC= c()

    for (h in 1:rep) {
    dat = Data(n,p,quant)
    y = dat$y
    u = dat$u
    x = dat$x
    g = x[,-1]
    u.grid = (1:200)*0.005
    gamma_0_grid = 2+2*sin(2*u.grid*pi)
    gamma_1_grid = 2*exp(2*u.grid-1)
    gamma_2_grid = -6*u.grid*(1-u.grid)
    gamma_3_grid = -4*u.grid^3
    coefficient = cbind(gamma_0_grid,gamma_1_grid,gamma_2_grid,gamma_3_grid)

    # a varying intercept not subject to regularization is automatically included by the package
    
    # BQRVCSS: Bayesian regularized quantile VC model with spike-and-slab priors (Zhou et al., CSDA, 2023)

    fit = pqrBayes(g, y, u, e=NULL, d = NULL, quant=quant, iterations=10000, burn.in = NULL, robust = TRUE, prior="SS", model = "VC", hyper=NULL,debugging=FALSE)
    coverage = coverage(fit,coefficient,u.grid, model = "VC")
    
    # BQRVC: Bayesian regularized quantile VC model (Zhou et al., CSDA, 2023)

    fit1 = pqrBayes(g, y, u, e=NULL, d =NULL,quant=quant, iterations=10000, burn.in = NULL, robust = TRUE, prior = "Laplace", model = "VC", hyper=NULL,debugging=FALSE)
    coverage1 = coverage(fit1,coefficient,u.grid, model = "VC")
    
    # BVCSS: Bayesian regularized VC model with spike-and-slab priors (Zhou et al., CSDA, 2023)
  
    fit2 = pqrBayes(g, y, u, e=NULL, d =NULL, quant=NULL, iterations=10000, burn.in = NULL, robust = FALSE, prior = "SS", model = "VC", hyper=NULL,debugging=FALSE)
    coverage2 = coverage(fit2,coefficient,u.grid, model = "VC")
    
    # BVC: Bayesian regularized VC model (Zhou et al., CSDA, 2023)
  
    fit3 = pqrBayes(g, y, u, e=NULL, d =NULL, quant=NULL, iterations=10000, burn.in = NULL, robust = FALSE, prior = "Laplace", model = "VC", hyper=NULL,debugging=FALSE)
    coverage3 = coverage(fit3,coefficient,u.grid,model = "VC")
    
    CI_BQRVCSS = rbind(CI_BQRVCSS,coverage)
    CI_BQRVC   = rbind(CI_BQRVC,coverage1)
    CI_BVCSS   = rbind(CI_BVCSS,coverage2)
    CI_BVC     = rbind(CI_BVC,coverage3)
    cat("Replicate = ", h, "\n")
    
    }
    # the varying intercept has not been regularized
    cp_BQRVCSS =  colMeans(CI_BQRVCSS) # 95% empirical coverage probabilities for the varying coefficients under the default setting
    cp_BQRVC   =  colMeans(CI_BQRVC)
    cp_BVCSS   =  colMeans(CI_BVCSS)
    cp_BVC     =  colMeans(CI_BVC)

## Example 4 (Bayesian Shrinkage Estimation for Robust Bayesian Group LASSO)

#### Data Generation for Group LASSO 

    Data <- function(n,p,quant){
      sig1 = matrix(0,p,p)
      diag(sig1)=1
      for (i in 1: p)
      {
      for (j in 1: p)
      {
      sig1[i,j]=0.5^abs(i-j)
      }
     }
    xx = MASS::mvrnorm(n,rep(0,p),sig1)
    x = cbind(1,xx)
    error0 = rt(n,2)
    error= error0 -quantile(error0,probs = quant) # can also be changed to normal error for non-robust setting
    beta = c(0,1,1.5,2,0,0,0,0.5,0.55,0.6,rep(0,p-9))
    betaa = beta[-1]
    y = x%*%beta+error
    dat = list(y=y, x=xx, beta=betaa)
    return(dat)
    }
#### robust Bayesian shrinkage with spike and slab prior estimation under group LASSO

    n=100; p=300;
    quant = 0.5; # focus on median for Bayesian estimation
    dat = Data(n,p,quant)
    y = dat$y
    g = dat$x
    coefficient = dat$beta

    # an intercept not subject to regularization is automatically included by the package

    # RBGLSS, (Ren et al. Biometrics, 2023)
    fit = pqrBayes(g, y, e=NULL, d=3, quant=quant, iterations=10000, burn.in = NULL,robust = TRUE, prior = "SS", model = "group", hyper=NULL,debugging=FALSE)
    
    fit$coefficients$GS.beta # posterior samples for regression coefficients 
    estimation_1 = estimation.pqrBayes(fit,coefficient,model="group")
    coeff_est_1 = estimation_1$coeff.est    
    mse_1 = estimation_1$error$MSE
    
    # RBGL: Bayesian Quantile Group LASSO (Li, Xi, & Lin, Bayesian Analysis, 2010)
    fit1 = pqrBayes(g, y, e=NULL,d=3, quant=quant, iterations=10000, burn.in = NULL, robust = TRUE, prior = "Laplace", model = "group", hyper=NULL,debugging=FALSE)
    
    fit1$coefficients$GS.beta # posterior samples for regression coefficients
    estimation_2 = estimation.pqrBayes(fit1,coefficient,model="group")
    coeff_est_2 = estimation_2$coeff.est    
    mse_2 = estimation_2$error$MSE 
    
    # BGLSS: Bayesian group LASSO with spike-and-slab priors (Xu & Ghosh, Bayesian Analysis, 2015) 
    fit2 = pqrBayes(g, y, d=3, e=NULL,quant=NULL, iterations=10000, burn.in = NULL, robust = FALSE, prior = "SS", model= "group", hyper=NULL,debugging=FALSE)
    estimation_3 = estimation.pqrBayes(fit2,coefficient,model="group")
    coeff_est_3 = estimation_3$coeff.est    
    mse_3 = estimation_3$error$MSE    
    
    # BGL: Bayesian group LASSO (Casella et al., Bayesian Analysis, 2010)
    fit3 = pqrBayes(g, y,d=3, e=NULL,quant=NULL, iterations=10000, burn.in = NULL, robust = FALSE, prior = "Laplace", model = "group", hyper=NULL,debugging=FALSE)
    estimation_4 = estimation.pqrBayes(fit3,coefficient,model="group")
    coeff_est_4 = estimation_4$coeff.est    
    mse_4 = estimation_4$error$MSE    

## Methods

This package provides implementation for methods from
  
  - Fan, K., Subedi, S., Yang, G., Lu, X., Ren, J. and Wu, C. (2024). Is Seeing Believing? A Practitioner's Perspective on High-dimensional Statistical Inference in Cancer Genomics Studies. [Entropy, 26(9),794](https://doi.org/10.3390/e26090794)
  - Zhou, F., Ren, J., Ma, S. and Wu, C. (2023). The Bayesian Regularized Quantile Varying Coefficient Model.  [Computational Statistics & Data Analysis, 187, 107808](https://doi.org/10.1016/j.csda.2023.107808)
  - Ren, J., Zhou, F., Li, X., Ma, S., Jiang, Y., and Wu, C. (2023). Robust Bayesian variable selection for gene–environment interactions. [Biometrics, 79(2), 684-694](https://doi.org/10.1111/biom.13670)
  - Fan, K. and Wu, C. (2025). A New Robust Binary Bayesian LASSO. [Stat, 14(3), e70078.](https://doi.org/10.1002/sta4.70078)
  - Fan, K., Srijana, S., Dissanayake, V. and Wu, C. (2026). Robust Bayesian high-dimensional variable selection and inference with the horseshoe family of priors. [Computational Statistics & Data Analysis, 219, 108358](https://doi.org/10.1016/j.csda.2026.108358)

pqrBayes_vc <- function(g, y, e=NULL,quant=0.5, iterations=10000, burn.in, robust=TRUE,prior="SS", hyper=NULL,debugging=FALSE){
  if(iterations<1) stop("iterations must be a positive integer.")
  if(is.null(burn.in)){
    BI = floor(iterations)/2
    if(iterations<=BI) stop("iterations must be larger than burn.in.")
  }else if(burn.in>=1){
    BI = as.integer(burn.in)
  }else{
    stop("burn.in must be a positive integer.")
  }
  n = length(y)
  u = stats::runif(n,0.01,0.99)
  kn=2
  degree=2
  # Check quant
  if (robust) {
    if (is.null(quant)) {
      stop("quant must be specified when robust = TRUE.")
    }
  } else {
    if (!is.null(quant)) {
      stop("quant must be NULL when robust = FALSE.")
    }
  }
  if(robust){
    out = Robust_vc(g, y, u, e,quant, iterations, kn, degree, prior, hyper,debugging)
  }else{
    out = NonRobust_vc(g, y, u, e,iterations, kn, degree, prior,debugging)
  }
  coefficient = list(GS.alpha=out$fit$GS.alpha[-c(1:BI),], GS.beta=out$fit$GS.beta[-c(1:BI),])
  fit = list(obj = out,coefficients = coefficient)
  return(fit)
}

pqrBayes_lin <- function(g, y, e, quant = 0.5, iterations = 10000, burn.in,
                         robust = TRUE, prior = "SS",
                         hyper = NULL, debugging = FALSE) {
  
  # Check iterations and burn-in
  if (iterations < 1) stop("iterations must be a positive integer.")
  
  if (is.null(burn.in)) {
    BI <- floor(iterations / 2)
    if (iterations <= BI) stop("iterations must be larger than burn.in.")
  } else if (burn.in >= 1) {
    BI <- as.integer(burn.in)
  } else {
    stop("burn.in must be a positive integer.")
  }
  
  # quant check
  if (robust) {
    if (is.null(quant)) {
      stop("quant must be specified when robust = TRUE.")
    }
  } else {
    if (!is.null(quant)) {
      stop("quant must be NULL when robust = FALSE.")
    }
  }
  
  # Call appropriate model
  if (robust) {
    out <- Robust_lin(g, y, e, quant, iterations, prior, hyper, debugging)
  } else {
    out <- NonRobust_lin(g, y, e, iterations, prior, debugging)
  }
  
  # Extract posterior samples
  coefficient <- list(
    GS.alpha = out$fit$GS.alpha[-c(1:BI), ],
    GS.beta  = out$fit$GS.beta[-c(1:BI), ]
  )
  
  fit <- list(obj = out, coefficients = coefficient)
  return(fit)
}

predict_vc <- function(object, g.new, e.new = NULL, y.new,
                        robust = TRUE, quant, ...) {
  
  if (robust) {
    
    if (is.null(quant)) {
      stop("quant must be specified when robust = TRUE.")
    }
    
    out <- predict_vc_robust(
      obj   = object,
      g.new = g.new,
      e.new = e.new,
      y.new = y.new,
      quant = quant,
      ...
    )
    
  } else {
    
    if (!is.null(quant)) {
      stop("quant must be NULL when robust = FALSE.")
    }
    out <- predict_vc_nonrobust(
      obj   = object,
      g.new = g.new,
      e.new = e.new,
      y.new = y.new,
      ...
    )
    
  }
  
  return(out)
}

estimation_lin = function(obj,coefficient){
  coeff.est = c()
  posterior = obj$coefficients
  for(j in 1:dim(posterior$GS.beta)[2])
  {
    t1=as.matrix(posterior$GS.beta[,j])
    coeff.est[j] = stats::median(t1)
  }
  MSE = mean((coeff.est-coefficient)^2)
  error.est = list(MSE = MSE)
  est_lin = list(error=error.est, coeff.est=coeff.est)
  return(est_lin)
}

predict_vc_nonrobust=function(obj, g.new, u.new, e.new, y.new,...){
  p = dim(g.new)[2]
  kn = obj$obj$kn
  degree = obj$obj$degree
  x = cbind(1,g.new)
  n = dim(g.new)[1]; 
  
  
  
  ## basis expansion
  
  d=kn+degree+1
  u.k = seq(0, 1, length=kn+2)[-c(1,kn+2)]
  Knots = as.numeric(stats::quantile(u.new, u.k))
  pi.u = splines::bs(u.new, knots=Knots, intercept=TRUE, degree=degree)[,1:(d)]
  
  
  
  # 50% quantile of posterior samples
  
  c1.C=rep(0,d)
  for (i in 1:d) {
    c1.C[i]=stats::quantile(obj$coefficients$GS.alpha[,i],0.5)
  }
  
  c2.C=rep(0,dim(obj$coefficients$GS.beta)[2])
  for (i in 1:dim(obj$coefficients$GS.beta)[2]) {
    c2.C[i]=stats::quantile(obj$coefficients$GS.beta[,i],0.5)
  }
  
  coeffmatrix.C=as.matrix(cbind(c1.C,matrix(c2.C,nrow = d)))
  
  gamma.hat=pi.u%*% coeffmatrix.C
  
  if(!is.null(e.new)){
    q = dim(e.new)[2]
    
    
    beta.hat=rep(0,q)
    for (i in 1:q) {
      beta.hat[i]=stats::quantile(obj$coefficients$GS.alpha[,i+d],0.5)
    }
    
    # prediction error
    y.hat=res=rep(0,n)
    for (i in 1:n) {
      y.hat[i]=t(x[i,])%*%gamma.hat[i,]+t(e.new[i,])%*%beta.hat
      
      res[i] = (y.new[i]-y.hat[i])^2
    }
    
    pred.error=mean(res)
    
    
  }else{
    
    # prediction error
    y.hat=res=rep(0,n)
    for (i in 1:n) {
      y.hat[i]=t(x[i,])%*%gamma.hat[i,]
      
      res[i] = (y.new[i]-y.hat[i])^2
    }
    
    pred.error=mean(res)
    
  }
  
  
  pqrBayes_vc_nonrobust= list(error=pred.error, y.pred=y.hat)
  #class(pqrBayes.pred) = "pqrBayes.pred"
  return(pqrBayes_vc_nonrobust)
  #pred
}

#' Make predictions from a pqrBayes object
#'
#' Make predictions from a pqrBayes object
#'
#' @param object a pqrBayes object.
#' @param g.new a matrix of new predictors (e.g. genetic factors) at which predictions are to be made. When being applied to the sparse linear, binary LASSO or group LASSO, g.new = g.
#' @param u.new a vector of new environmental factor at which predictions are to be made. When being applied to the sparse linear model, binary LASSO or group LASSO, u.new = NULL. The default value is NULL.
#' @param e.new a vector or matrix of new clinical covariates at which predictions are to be made. The default value is NULL.
#' @param y.new a vector of the response of new observations. When being applied to the sparse linear model, binary LASSO or group LASSO, y.new = y.
#' @param robust logical flag. If TRUE, robust methods are used. Otherwise, non-robust methods are used. The default value is TRUE.
#' @param quant the quantile level specified by users. Required when robust = TRUE. Ignored (set to NULL) when robust = FALSE.The default value is 0.5.
#' @param model the model to be fitted. The default is "VC" for a quantile varying coefficient model. Users can also specify "linear" for a sparse linear model, "binary" for binary LASSO and "group" for a group LASSO.
#' @param ... other predict arguments
#' 
#' @details g.new (u.new) must have the same number of columns as g (u) used for fitting the model. By default, the clinical covariates are NULL unless 
#' provided. The predictions are made based on the posterior estimates of coefficients in the pqrBayes object.
#'
#'
#' @usage predict_pqrBayes(object, g.new, u.new, e.new, y.new, robust, quant, model, ...)
#' @return  an object of class `pqrBayes.pred' is returned, which is a list with components:
#' \item{error}{prediction error.}
#' \item{y.pred}{predicted values of the new observations.}
#'
#' @rdname predict_pqrBayes
#' @seealso \code{\link{pqrBayes}}
#' @examples
#' ## The quantile regression model
#' data(data)
#' data = data$data_linear
#' g=data$g
#' y=data$y
#' e=data$e
#' fit1=pqrBayes(g,y,e,model="linear")
#' prediction=predict_pqrBayes(fit1,g, y.new = y, model="linear")
#' @export
predict_pqrBayes=function(object, g.new, u.new=NULL, e.new=NULL, y.new, robust=T, quant=0.5,model,...){
  if (model != "VC" && !is.null(u.new)) {
    
    stop("Argument 'u.new' should be NULL unless model = 'VC'.")
    
  }
  
  if (model == "VC" && is.null(u.new)) {
    
    stop("Argument 'u.new' must be provided when model = 'VC'.")
    
  }
  # quant check
  if (robust) {
    if (is.null(quant)) {
      stop("quant must be specified when robust = TRUE.")
    }
  } else {
    if (!is.null(quant)) {
      stop("quant must be NULL when robust = FALSE.")
    }
  }
  if(model=="VC"){
    pqrBayes.pred = predict_vc(object, g.new, u.new, e.new, y.new, robust, quant,...)
  }else if(model=="linear"){
    pqrBayes.pred = predict_lin(object, g.new, e.new, y.new, robust, quant,...)
  }else if(model=="binary"){
    pqrBayes.pred = predict_bin(object, g.new, e.new, y.new,...)
  }else if(model=="group"){
    pqrBayes.pred = predict_lin(object, g.new, e.new, y.new, robust, quant,...)
  }
  
  else{
    stop("model should be either VC, linear, binary or group")
  }
  class(pqrBayes.pred) = "pqrBayes.pred"
  return(pqrBayes.pred)
  #pred
}

Robust_g <- function(g, y, e, d, quant, iterations, prior, hyper,debugging){
  
  p = dim(g)[2]
  
  x = cbind(1,g)
  n = length(y); 
  
  if(!is.null(e)){
    q = dim(e)[2]
    invSigAlpha0 = diag(10^-3, (1+q))
    hatAlpha = rep(1,(1+q))
    w = cbind(1,e)
  }else{
    q = 0
    hatAlpha=rep(1,1)
    invSigAlpha0 = diag(10^-3, 1)
    w = matrix(1,n,1)
  }
  
  xi1 = (1-2*quant)/(quant*(1-quant))
  xi2 = sqrt(2/(quant*(1-quant)))
  hatBeta= matrix(rep(1,p),ncol = p/d)
  hatSg=rep(1,p/d)
  hatTau = 1
  hatV = rep(1,n) 
  hatEtaSq = 1
  hatPi = 0.5
  
  sh0_1 = ifelse(is.null(hyper$a0), 1, hyper$a0)
  sh0_0 = ifelse(is.null(hyper$b0), 1, hyper$b0)
  
  a = ifelse(is.null(hyper$c1), 1, hyper$c1)
  b = ifelse(is.null(hyper$c2), 1, hyper$c2)
  
  r = ifelse(is.null(hyper$d2), 1, hyper$d2)
  
  progress = ifelse(debugging, 10^(floor(log10(iterations))-1), 0)
  if(prior== "SS"){
    fit=BRGL_SS(g, y, w, p/d, d, iterations, hatAlpha, hatBeta, hatTau, hatV, hatSg, invSigAlpha0, hatPi, hatEtaSq,
                xi1, xi2, r, a, b, sh0_1, sh0_0, progress)
  }
  else if(prior=="Laplace"){fit= BRGL(g, y, w, p/d, d, iterations, hatAlpha, hatBeta, hatTau, hatV, hatSg, invSigAlpha0, hatEtaSq,
                 xi1, xi2, r, a, b, progress)
  }
  else{stop("The specified prior is currently not supported.")}
  out = list(fit=fit,iterations=iterations)
  return(out)
}

Robust_vc <- function(g, y, u, e,quant, iterations, kn, degree,prior, hyper,debugging){
  
  p = dim(g)[2]
  
  x = cbind(1,g)
  n = length(y); 
  
  
  
  ## basis expansion
  
  d=kn+degree+1
  u.k = seq(0, 1, length=kn+2)[-c(1,kn+2)]
  Knots = as.numeric(stats::quantile(u, u.k))
  pi.u = splines::bs(u, knots=Knots, intercept=TRUE, degree=degree)[,1:(d)]
  
  
  xx = as.data.frame(matrix(0, n, (p+1)*d))
  for(j in 1:(p+1)){
    last = j*d; first = last-d+1
    xx[,first:last] = pi.u*x[,j]
  }
  xx = as.matrix(xx)
  
  if(!is.null(e)){
    q = dim(e)[2]
    xxwe = cbind(xx, e)
    lasso.cv = glmnet::cv.glmnet(xxwe,y,alpha=1,nfolds=5)
    lambda.cv = lasso.cv$lambda.min;
    lasso.fit = glmnet::glmnet(xxwe, y, family="gaussian",alpha=1,nlambda=50)
    coeff.array = as.vector(stats::predict(lasso.fit, s=lambda.cv, type="coefficients"))[-1];
    
    hat.m = coeff.array[1:d]      ## coeff for varying intercept
    hat.r = coeff.array[(d+1):((p+1)*d)] ## coeff for varying part
    hat.clin = coeff.array[((p+1)*d+1):dim(xxwe)[2]] ## coeff for clinic covariates
    
    
    xx1=xx[,-(1:d)]
    CLC=cbind(pi.u,e)
    hatAlpha=c(hat.m,hat.clin)
    invSigAlpha0 = diag(10^-3, (d+q))
  }else{
    q = 0
    xxwe = xx
    lasso.cv = glmnet::cv.glmnet(xxwe,y,alpha=1,nfolds=5)
    lambda.cv = lasso.cv$lambda.min;
    lasso.fit = glmnet::glmnet(xxwe, y, family="gaussian",alpha=1,nlambda=50)
    coeff.array = as.vector(stats::predict(lasso.fit, s=lambda.cv, type="coefficients"))[-1];
    
    hat.m = coeff.array[1:d]      ## coeff for varying intercept
    hat.r = coeff.array[(d+1):((p+1)*d)] ## coeff for varying part
    
    
    xx1=xx[,-(1:d)]
    CLC=pi.u
    hatAlpha=hat.m
    invSigAlpha0 = diag(10^-3, (d+q))
  }
  
  xi1 = (1-2*quant)/(quant*(1-quant))
  xi2 = sqrt(2/(quant*(1-quant)))
  hatTau = 1
  hatV = rep(1,n) 
  hatEtaSq = 1
  hatSg = rep(1, p)
  hatPi = 0.5
  
  sh0_1 = ifelse(is.null(hyper$a0), 1, hyper$a0)
  sh0_0 = ifelse(is.null(hyper$b0), 1, hyper$b0)
  
  a = ifelse(is.null(hyper$c1), 1, hyper$c1)
  b = ifelse(is.null(hyper$c2), 1, hyper$c2)
  
  r = ifelse(is.null(hyper$d2), 1, hyper$d2)
  hatbeta=matrix(hat.r,ncol=p) + 10^-5
  progress = ifelse(debugging, 10^(floor(log10(iterations))-1), 0)
  if(prior=="SS"){fit=BRGL_SS(xx1, y, CLC, p, d, iterations, hatAlpha, hatbeta, hatTau, hatV, hatSg, invSigAlpha0, hatPi, hatEtaSq,
                         xi1, xi2, r, a, b, sh0_1, sh0_0, progress)}
  else if (prior=="Laplace"){fit=BRGL(xx1, y, CLC, p, d, iterations, hatAlpha, hatbeta, hatTau, hatV, hatSg, invSigAlpha0, hatEtaSq,
                xi1, xi2, r, a, b, progress)}
  else{stop("The specified prior is currently not supported.")}
  out = list(fit = fit,kn = kn,degree = degree,iterations = iterations)
  return(out)
}

VCselect <- function(obj,prior){
  if(prior=="SS"){
   
    idgene=obj$obj$idgene
    iterations = obj$obj$iterations
    id=which(idgene>iterations/4)
    VCselect=list(id=id,idgene = idgene)
  }else{
    
    kn = obj$obj$kn
    degree = obj$obj$degree
    d=kn+degree+1
    iterations = obj$obj$iterations
    # 2.5% quantile of posterior samples
    
    c2.C=rep(0,dim(obj$coefficients$GS.beta)[2])
    for (i in 1:dim(obj$coefficients$GS.beta)[2]) {
      c2.C[i]=stats::quantile(obj$coefficients$GS.beta[,i],0.025)
    }
    
    coeffmatrix.C1=matrix(c2.C,nrow = d)
    
    # 97.5% quantile of posterior samples
    
    c2.C=rep(0,dim(obj$coefficients$GS.beta)[2])
    for (i in 1:dim(obj$coefficients$GS.beta)[2]) {
      c2.C[i]=stats::quantile(obj$coefficients$GS.beta[,i],0.975)
    }
    
    coeffmatrix.C2=matrix(c2.C,nrow = d)
    
    idgene=rep(0,dim(coeffmatrix.C1)[2])
    for (i in 1:dim(coeffmatrix.C1)[2]) {
      for (j in 1:d) {
        if(coeffmatrix.C1[j,i]*coeffmatrix.C2[j,i]>0){idgene[i]=1}
      }
    }
    id=which(idgene==1)
    VCselect=list(id=id,idgene = idgene)
  }
  
  #class(VCselect)="VCselect"
  return(VCselect)
  
}

estimation_vc = function(obj,coefficient,u.grid){
  posterior = obj$coefficients
  kn = obj$obj$kn
  degree = obj$obj$degree
  d=kn+degree+1
  u.star = seq(0, 1, length=kn+2)[-c(1,kn+2)]
  Knots.star = as.numeric(stats::quantile(u.grid, u.star))
  pi.star = splines::bs(u.grid, knots=Knots.star, intercept=TRUE, degree=degree)[,1:(d)]
  # 2.5% quantile of posterior samples
  
  c1_25.C=rep(0,d)
  for (i in 1:d) {
    c1_25.C[i]=stats::quantile(posterior$GS.alpha[,i],0.025)
  }
  
  c2_25.C=rep(0,dim(posterior$GS.beta)[2])
  for (i in 1:dim(posterior$GS.beta)[2]) {
    c2_25.C[i]=stats::quantile(posterior$GS.beta[,i],0.025)
  }
  
  coeffmatrix.C1_25=as.matrix(cbind(c1_25.C,matrix(c2_25.C,nrow = d)))
  
  
  # 97.5% quantile of posterior samples
  
  c1_975.C=rep(0,d)
  for (i in 1:d) {
    c1_975.C[i]=stats::quantile(posterior$GS.alpha[,i],0.975)
  }
  
  c2_975.C=rep(0,dim(posterior$GS.beta)[2])
  for (i in 1:dim(posterior$GS.beta)[2]) {
    c2_975.C[i]=stats::quantile(posterior$GS.beta[,i],0.975)
  }
  
  coeffmatrix.C2_975=as.matrix(cbind(c1_975.C,matrix(c2_975.C,nrow = d)))
  
  # 50% quantile of posterior samples
  
  c1.C=rep(0,d)
  for (i in 1:d) {
    c1.C[i]=stats::quantile(posterior$GS.alpha[,i],0.5)
  }
  
  c2.C=rep(0,dim(posterior$GS.beta)[2])
  for (i in 1:dim(posterior$GS.beta)[2]) {
    c2.C[i]=stats::quantile(posterior$GS.beta[,i],0.5)
  }
  
  coeffmatrix.C=as.matrix(cbind(c1.C,matrix(c2.C,nrow = d)))
  
  gamma.hat=pi.star%*% coeffmatrix.C[,1:ncol(coefficient)]
  gamma.hat_25 = pi.star%*% coeffmatrix.C1_25[,1:ncol(coefficient)]
  gamma.hat_975 = pi.star%*% coeffmatrix.C2_975[,1:ncol(coefficient)]
  IMSE = rep(0,ncol(coefficient))
  for(i in 1:ncol(coefficient)){
    IMSE[i] = mean((gamma.hat[,i]-coefficient[,i])^2)
  }
  TIMSE = sum(IMSE)
  error.est = list(IMSE = IMSE,TIMSE = TIMSE)
  coeff.est = list(gamma.hat = gamma.hat,gamma.hat_25 = gamma.hat_25,gamma.hat_975 = gamma.hat_975)
  est_vc = list(error=error.est, coeff.est=coeff.est)
  return(est_vc)
}

coverage_lin = function(obj,coefficient,u.grid=NULL){
  C = c()
  for(j in 1:dim(obj$coefficients$GS.beta)[2])
  {
    t1=as.matrix(obj$coefficients$GS.beta[,j])
    c_j = as.matrix(stats::quantile(t1,c(0.025,0.975)))
    C[j] = ifelse(coefficient[j]>=c_j[1]&coefficient[j]<=c_j[2],1,0)
  }
  
  return(C)
}

linselect <- function(obj,prior){
  if(prior=="SS"){
    mpm <- function(x)
    {
      if (mean(x) >= 0.5) {1}
      else {0}
    }
    sg1 = obj$coefficients$GS.beta
    sg1[which(sg1!=0)]=1
    id=c()
    for(j in 1:ncol(sg1)){
      t1=as.matrix(sg1[,j])
      q_t1 = mpm(t1)
      id = c(id,q_t1)
    }
    linselect=list(id=id)
  }else{
    fun <- function(x)
    {
      pp = prod(x)
      if(sign(pp)==1) {1}
      else {0}
    }
    sg1 = obj$coefficients$GS.beta
    q_1=c()
    for(j in 1:ncol(sg1)){
      t1=as.matrix(sg1[,j])
      q_t1 = as.matrix(stats::quantile(t1,c(0.025,0.975)))
      q_1 = cbind(q_1,q_t1)
    }
    id = apply(q_1, 2, fun)
    
    linselect=list(id=id)
  }
  
  #class(VCselect)="VCselect"
  return(linselect)
  
}

Robust_lin <- function(g, y, e, quant, iterations,prior, hyper,debugging){
 
  p = dim(g)[2]
  
  x = cbind(1,g)
  n = length(y); 
  
  if(!is.null(e)){
    q = dim(e)[2]
    invSigAlpha0 = diag(10^-3, (1+q))
    hatAlpha = rep(1,(1+q))
    w = cbind(1,e)
  }else{
    q = 0
    hatAlpha=rep(1,1)
    invSigAlpha0 = diag(10^-3, 1)
    w = matrix(1,n,1)
  }
  
  xi1 = (1-2*quant)/(quant*(1-quant))
  xi2 = sqrt(2/(quant*(1-quant)))
  hatBeta=rep(1,p)
  hatSg=rep(1,p)
  hatTau = 1
  hatV = rep(1,n) 
  hatEtaSq = 1
  hatPi = 0.5
  
  sh0_1 = ifelse(is.null(hyper$a0), 1, hyper$a0)
  sh0_0 = ifelse(is.null(hyper$b0), 1, hyper$b0)
  
  a = ifelse(is.null(hyper$c1), 1, hyper$c1)
  b = ifelse(is.null(hyper$c2), 1, hyper$c2)
  
  r = ifelse(is.null(hyper$d2), 1, hyper$d2)
  hatTauSq=1
  hatcSq = 1
  hatlamSq = rep(1,p)
  hatZeta = 1
  hatnu = rep(1,p)
  hatnu_1 = rep(1,p)
  hatnu_2 = rep(1,p)
  hatetaSq = rep(1,p)
  hatSigma = 1
  hatSigmaSq = 1
  progress=0
  c = 0.002
  d = 90
  progress = ifelse(debugging, 10^(floor(log10(iterations))-1), 0)
  if(prior == "SS"){
    fit=BRLSS(g, y, w, iterations, hatAlpha, hatBeta, hatTau, hatV, hatSg, invSigAlpha0, hatPi, hatEtaSq, xi1, xi2, r, a, b,sh0_1,sh0_0, progress)}
  else if (prior =="HS"){
    fit = RBhorse (g, y, w, iterations, hatAlpha, hatBeta, hatSigma, hatV, hatSg, hatnu, invSigAlpha0, hatTauSq, xi1, xi2, hatZeta, a, b, progress)
  }
  else if (prior =="HS+" ){
    fit = RBhorse_plus(g, y, w, iterations, hatAlpha, hatBeta, hatSigma, hatV, hatSg, hatetaSq, hatnu_1,hatnu_2, invSigAlpha0, hatTauSq, xi1, xi2, hatZeta, a, b, progress)
  }
  else if (prior =="RHS"){
    fit = RBhorse_reg (g, y, w, iterations, hatAlpha, hatBeta, hatSigma, hatcSq, hatV, hatSg, hatnu, invSigAlpha0, hatTauSq, xi1, xi2, hatZeta, a, b, c, d, progress)
  }
  else if(prior == "Laplace"){fit= BRL(g, y, w, iterations, hatAlpha, hatBeta, hatTau, hatV, hatSg, invSigAlpha0, hatEtaSq, xi1, xi2, r, a, b, progress)}
  else{stop("The specified prior is currently not supported.")}
  out = list(fit=fit,iterations=iterations)
  return(out)
}

#' Simulated data under high-dimensional linear, binary, group LASSO and quantile varying coefficient models
#'
#' @docType data
#' @keywords datasets
#' @name data
#' @format 
#' The data_linear object consists of 4 components: g, y, e and coeff. coeff contains the true values of parameters used for generating the response variable \eqn{y}.
#' The data_binary object consists of 4 components: g, y, e and coeff. coeff contains the true values of parameters used for generating the response variable \eqn{y}.
#' The data_group object consists of 4 components: g, y, e and coeff. coeff contains the true values of parameters used for generating the response variable \eqn{y}.
#' The data_varying object consists of five components: g, y, u, e and coeff. coeff contains the true values of parameters used for generating the response variable \eqn{y}.
#' 
#'
#' 
#'
#' @details
#'
#' \strong{Generating Y using a sparse linear (quantile) regression model}
#' 
#' The true data generating model under sparse linear regression:
#' \deqn{Y_i=\beta_0+\beta_{1}X_{i1}+\beta_{2}X_{i2}+\beta_{3}X_{i3}+\epsilon_i,}
#' where \eqn{\epsilon_i\sim N(0,1)}, \eqn{\beta_{0}=0}, \eqn{\beta_{1}=1 }, \eqn{\beta_{2}=1.5} and \eqn{\beta_3=2}.
#' 
#' \strong{Generating Y using a sparse binary (quantile) regression model}
#' 
#' The true data generating model under sparse linear regression:
#' \deqn{\tilde{Y}_i=\beta_0+\beta_{1}X_{i1}+\beta_{2}X_{i2}+\beta_{3}X_{i3}+\epsilon_i,}
#' where \eqn{\epsilon_i\sim N(0,1)}, \eqn{\beta_{0}=0}, \eqn{\beta_{1}=0.22 }, \eqn{\beta_{2}=0.18} and \eqn{\beta_3=0.14}.
#' 
#' \eqn{Y_i=1} if \eqn{\tilde{Y}_i>0} and \eqn{Y_i=0} otherwise.
#' 
#' \strong{Generating Y using a high-dimensional group LASSO model}
#' 
#' The true data generating model under a group LASSO model:
#' \deqn{Y_i=\beta_0+\beta_{1}X_{i1}+\beta_{2}X_{i2}+\beta_{3}X_{i3}+\beta_{7}X_{i7}+\beta_{8}X_{i8}+\beta_{9}X_{i9}+\epsilon_i,}
#' where \eqn{\epsilon_i\sim N(0,1)}, \eqn{\beta_{0}=0}, \eqn{\beta_{1}=0.6}, \eqn{\beta_{2}=0.7},\eqn{\beta_{3}=0.8},\eqn{\beta_{7}=0.65}, \eqn{\beta_{8}=0.75} and \eqn{\beta_{9}=0.85}.
#' 
#' \strong{Generating Y using a (quantile) varying coefficient model}
#'
#' Data generation under sparse (quantile) VC model:
#' \deqn{Y_i=\gamma_0(v_i)+\gamma_1(v_i)X_{i1}+\gamma_2(v_i)X_{i2}+\gamma_3(v_i)X_{i3}+\epsilon_i,}
#' where \eqn{\epsilon_i\sim N(0,1)}, \eqn{\gamma_{0}(v_i)=1.5\sin(0.2\pi*v_i}), \eqn{\gamma_{1}(v_i)=2\exp(0.2v_i-1)-1.5 }, \eqn{\gamma_{2}(v_i)=2-2v_i } and \eqn{\gamma_3(v_i)=-4+(v_i-2)^3/6}.
#'
#' @examples
#' data(data)
#' data = data$data_linear
#' g=data$g
#' dim(g)
#' y=data$y
#' coeff=data$coeff
#' print(coeff)
#' 
#' data = data$data_binary
#' g=data$g
#' dim(g)
#' y=data$y
#' coeff=data$coeff
#' print(coeff)
#' 
#' data = data$data_group
#' g=data$g
#' dim(g)
#' y=data$y
#' coeff=data$coeff
#' print(coeff)
#' 
#' data = data$data_varying
#' g=data$g
#' dim(g)
#' coeff=data$coeff
#' print(coeff)
#'
#'
#' @seealso \code{\link{pqrBayes}}
NULL

coverage_vc = function(obj,coefficient,u.grid){
  kn = obj$obj$kn
  degree = obj$obj$degree
  d=kn+degree+1
  u.star = seq(0, 1, length=kn+2)[-c(1,kn+2)]
  Knots.star = as.numeric(stats::quantile(u.grid, u.star))
  pi.star = splines::bs(u.grid, knots=Knots.star, intercept=TRUE, degree=degree)[,1:(d)]
  # 2.5% quantile of posterior samples
  
  c1_25.C=rep(0,d)
  for (i in 1:d) {
    c1_25.C[i]=stats::quantile(obj$coefficients$GS.alpha[,i],0.025)
  }
  
  c2_25.C=rep(0,dim(obj$coefficients$GS.beta)[2])
  for (i in 1:dim(obj$coefficients$GS.beta)[2]) {
    c2_25.C[i]=stats::quantile(obj$coefficients$GS.beta[,i],0.025)
  }
  
  coeffmatrix.C1_25=as.matrix(cbind(c1_25.C,matrix(c2_25.C,nrow = d)))
  
  
  # 97.5% quantile of posterior samples
  
  c1_975.C=rep(0,d)
  for (i in 1:d) {
    c1_975.C[i]=stats::quantile(obj$coefficients$GS.alpha[,i],0.975)
  }
  
  c2_975.C=rep(0,dim(obj$coefficients$GS.beta)[2])
  for (i in 1:dim(obj$coefficients$GS.beta)[2]) {
    c2_975.C[i]=stats::quantile(obj$coefficients$GS.beta[,i],0.975)
  }
  
  coeffmatrix.C2_975=as.matrix(cbind(c1_975.C,matrix(c2_975.C,nrow = d)))
  gamma.var1 = pi.star%*% coeffmatrix.C1_25[,1:ncol(coefficient)]
  gamma.var2 = pi.star%*% coeffmatrix.C2_975[,1:ncol(coefficient)]
  CI = matrix(,nrow(coefficient),ncol(coefficient))
  for(i in 1:ncol(coefficient)){
    CI[,i] = ifelse(coefficient[,i]>=gamma.var1[,i]&coefficient[,i]<=gamma.var2[,i],1,0)
  }
  C = apply(CI, 2, function(col) ifelse(all(col == 1), 1, 0))
  return(C)
}

#' Variable selection for a pqrBayes object
#'
#' Variable selection for a pqrBayes object
#'
#' @param object a pqrBayes object.
#' @param prior the prior used in the pqrBayes function. Users can choose "SS" for the spike-and-slab prior, "HS" for the horseshoe prior, "HS+" for the horseshoe plus prior, "RHS" for the regularized horseshoe prior and "Laplace" for the Laplace prior. The default value is "SS".
#' @param model the model to be fitted. Users can also choose "linear" for a sparse linear model (with a continuous response), "VC" for a varying coefficient model or "group" for group LASSO.
#'
#' @details For class `Sparse', the median probability model (MPM) (Barbieri and Berger, 2004) is used to identify predictors that are significantly associated
#' with the response variable. For class `NonSparse', variable selection is based on 95\% credible interval.
#' Please check the references for more details about the variable selection.
#'
#' @references
#' Ren, J., Zhou, F., Li, X., Ma, S., Jiang, Y. and Wu, C. (2023). Robust Bayesian variable selection for gene-environment interactions. 
#' {\emph{Biometrics}, 79(2), 684-694} \doi{10.1111/biom.13670}
#'
#' Barbieri, M.M. and Berger, J.O. (2004). Optimal predictive model selection. {\emph{Ann. Statist}, 32(3):870–897}
#'
#' @rdname select.pqrBayes
#' @usage pqrBayes.select(object,prior="SS",model="linear")
#' @return an object of class `select' is returned, which includes the indices of the selected predictors (e.g. genetic factors).
#'
#' @seealso \code{\link{pqrBayes}}
#'
#' @examples
#' ## The sparse quantile regression model
#' data(data)
#' data = data$data_linear
#' g=data$g
#' y=data$y
#' e=data$e
#' 
#' fit1=pqrBayes(g,y,e,d = NULL,quant=0.5,model="linear")
#' select=pqrBayes.select(obj = fit1,prior = "SS",model="linear")
#' 
#' ## The quantile varying coefficient model
#' data(data)
#' data = data$data_varying
#' g=data$g
#' y=data$y
#' e=data$e
#' fit1=pqrBayes(g,y,e,d = NULL,quant=0.5,model="VC")
#' select=pqrBayes.select(obj = fit1,prior = "SS",model="VC")
#' select
#'
#' \donttest{
#' ## Non-sparse example with VC model
#' fit2 <- pqrBayes(
#'   g = g, y = y, e = e, d = NULL,
#'   quant = 0.5,
#'   prior= "Laplace",
#'   model = "VC"
#' )
#'
#' select <- pqrBayes.select(obj = fit2, prior = "SS", model = "VC")
#' select
#' }

#'
#' @export
pqrBayes.select <- function(object,prior="SS",model="linear"){
  if(model=="VC"){
    select = VCselect(obj = object,prior)
  }
  else if(model=="linear"){
    select = linselect(obj = object,prior)
  }
  else if(model=="binary"){
    select = linselect(obj = object,prior)
  }
  else if(model=="group"){
    select = linselect(obj = object,prior)
  }
  else{
    stop("model should be either VC, linear, binary or group")
  }
  class(select)="select"
  return(select)
}
  
  
 

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

BBL <- function(xx, y, ystar, W, maxSteps, hatBeta, hatAlpha, hatInvTauSq, invSigAlpha0, hatLambdaSqStar, hatSigmaSq, aStar, bStar, alpha, gamma, progress) {
    .Call(`_pqrBayes_BBL`, xx, y, ystar, W, maxSteps, hatBeta, hatAlpha, hatInvTauSq, invSigAlpha0, hatLambdaSqStar, hatSigmaSq, aStar, bStar, alpha, gamma, progress)
}

BBLSS <- function(xx, y, ystar, W, maxSteps, hatAlpha, hatBeta, hatInvTauSq, invSigAlpha0, hatPi, hatLambdaSq, hatSigmaSq, aStar, bStar, alpha, gamma, sh1, sh0, progress) {
    .Call(`_pqrBayes_BBLSS`, xx, y, ystar, W, maxSteps, hatAlpha, hatBeta, hatInvTauSq, invSigAlpha0, hatPi, hatLambdaSq, hatSigmaSq, aStar, bStar, alpha, gamma, sh1, sh0, progress)
}

BGLPointMass <- function(xx, y, W, s, q, maxSteps, hatAlpha, hatBeta, hatInvTauSqStar, invSigAlpha0, hatPiStar, hatLambdaSqStar, hatSigmaSq, aStar, bStar, alpha, gamma, sh1, sh0, progress) {
    .Call(`_pqrBayes_BGLPointMass`, xx, y, W, s, q, maxSteps, hatAlpha, hatBeta, hatInvTauSqStar, invSigAlpha0, hatPiStar, hatLambdaSqStar, hatSigmaSq, aStar, bStar, alpha, gamma, sh1, sh0, progress)
}

BGL <- function(xx, y, W, s, q, maxSteps, hatBeta, hatAlpha, hatInvTauSq, invSigAlpha0, hatLambdaSqStar, hatSigmaSq, aStar, bStar, alpha, gamma, progress) {
    .Call(`_pqrBayes_BGL`, xx, y, W, s, q, maxSteps, hatBeta, hatAlpha, hatInvTauSq, invSigAlpha0, hatLambdaSqStar, hatSigmaSq, aStar, bStar, alpha, gamma, progress)
}

BL <- function(xx, y, W, maxSteps, hatBeta, hatAlpha, hatInvTauSq, invSigAlpha0, hatLambdaSqStar, hatSigmaSq, aStar, bStar, alpha, gamma, progress) {
    .Call(`_pqrBayes_BL`, xx, y, W, maxSteps, hatBeta, hatAlpha, hatInvTauSq, invSigAlpha0, hatLambdaSqStar, hatSigmaSq, aStar, bStar, alpha, gamma, progress)
}

BLSS <- function(xx, y, W, maxSteps, hatAlpha, hatBeta, hatInvTauSq, invSigAlpha0, hatPi, hatLambdaSq, hatSigmaSq, aStar, bStar, alpha, gamma, sh1, sh0, progress) {
    .Call(`_pqrBayes_BLSS`, xx, y, W, maxSteps, hatAlpha, hatBeta, hatInvTauSq, invSigAlpha0, hatPi, hatLambdaSq, hatSigmaSq, aStar, bStar, alpha, gamma, sh1, sh0, progress)
}

BRL <- function(xx, y, W, maxSteps, hatAlpha, hatBeta, hatTau, hatV, hatSg, invSigAlpha0, hatEtaSq, xi1, xi2, r1, a, b, progress) {
    .Call(`_pqrBayes_BRL`, xx, y, W, maxSteps, hatAlpha, hatBeta, hatTau, hatV, hatSg, invSigAlpha0, hatEtaSq, xi1, xi2, r1, a, b, progress)
}

BRLSS <- function(xx, y, W, maxSteps, hatAlpha, hatBeta, hatTau, hatV, hatSg, invSigAlpha0, hatPi, hatEtaSq, xi1, xi2, r1, a, b, sh1, sh0, progress) {
    .Call(`_pqrBayes_BRLSS`, xx, y, W, maxSteps, hatAlpha, hatBeta, hatTau, hatV, hatSg, invSigAlpha0, hatPi, hatEtaSq, xi1, xi2, r1, a, b, sh1, sh0, progress)
}

BRBL <- function(xx, y, ystar, W, maxSteps, hatAlpha, hatBeta, hatTau, hatV, hatSg, invSigAlpha0, hatEtaSq, xi1, xi2, r1, a, b, progress) {
    .Call(`_pqrBayes_BRBL`, xx, y, ystar, W, maxSteps, hatAlpha, hatBeta, hatTau, hatV, hatSg, invSigAlpha0, hatEtaSq, xi1, xi2, r1, a, b, progress)
}

BRBLSS <- function(xx, y, ystar, W, maxSteps, hatAlpha, hatBeta, hatTau, hatV, hatSg, invSigAlpha0, hatPi, hatEtaSq, xi1, xi2, r1, a, b, sh1, sh0, progress) {
    .Call(`_pqrBayes_BRBLSS`, xx, y, ystar, W, maxSteps, hatAlpha, hatBeta, hatTau, hatV, hatSg, invSigAlpha0, hatPi, hatEtaSq, xi1, xi2, r1, a, b, sh1, sh0, progress)
}

BRGL_SS <- function(xx, y, W, s, L, maxSteps, hatAlpha, hatBeta, hatTau, hatV, hatSg, invSigAlpha0, hatPi, hatEtaSq, xi1, xi2, r, a, b, sh1, sh0, progress) {
    .Call(`_pqrBayes_BRGL_SS`, xx, y, W, s, L, maxSteps, hatAlpha, hatBeta, hatTau, hatV, hatSg, invSigAlpha0, hatPi, hatEtaSq, xi1, xi2, r, a, b, sh1, sh0, progress)
}

BRGL <- function(xx, y, W, s, L, maxSteps, hatAlpha, hatBeta, hatTau, hatV, hatSg, invSigAlpha0, hatEtaSq, xi1, xi2, r, a, b, progress) {
    .Call(`_pqrBayes_BRGL`, xx, y, W, s, L, maxSteps, hatAlpha, hatBeta, hatTau, hatV, hatSg, invSigAlpha0, hatEtaSq, xi1, xi2, r, a, b, progress)
}

RBhorse_plus <- function(xx, y, W, maxSteps, hatAlpha, hatBeta, hatSigma, hatV, hatSg, hatetaSq, hatnu_1, hatnu_2, invSigAlpha0, hatTauSq, xi1, xi2, hatZeta, a, b, progress) {
    .Call(`_pqrBayes_RBhorse_plus`, xx, y, W, maxSteps, hatAlpha, hatBeta, hatSigma, hatV, hatSg, hatetaSq, hatnu_1, hatnu_2, invSigAlpha0, hatTauSq, xi1, xi2, hatZeta, a, b, progress)
}

RBhorse <- function(xx, y, W, maxSteps, hatAlpha, hatBeta, hatSigma, hatV, hatSg, hatnu, invSigAlpha0, hatTauSq, xi1, xi2, hatZeta, a, b, progress) {
    .Call(`_pqrBayes_RBhorse`, xx, y, W, maxSteps, hatAlpha, hatBeta, hatSigma, hatV, hatSg, hatnu, invSigAlpha0, hatTauSq, xi1, xi2, hatZeta, a, b, progress)
}

RBhorse_reg <- function(xx, y, W, maxSteps, hatAlpha, hatBeta, hatSigma, hatcSq, hatV, hatSg, hatnu, invSigAlpha0, hatTauSq, xi1, xi2, hatZeta, a, b, c, d, progress) {
    .Call(`_pqrBayes_RBhorse_reg`, xx, y, W, maxSteps, hatAlpha, hatBeta, hatSigma, hatcSq, hatV, hatSg, hatnu, invSigAlpha0, hatTauSq, xi1, xi2, hatZeta, a, b, c, d, progress)
}

Bhorse_plus <- function(xx, y, W, maxSteps, hatBeta, hatAlpha, hatlamSq, hatetaSq, hatnu_1, hatnu_2, invSigAlpha0, hatTauSq, hatSigmaSq, hatZeta, a, b, progress) {
    .Call(`_pqrBayes_Bhorse_plus`, xx, y, W, maxSteps, hatBeta, hatAlpha, hatlamSq, hatetaSq, hatnu_1, hatnu_2, invSigAlpha0, hatTauSq, hatSigmaSq, hatZeta, a, b, progress)
}

Bhorse <- function(xx, y, W, maxSteps, hatBeta, hatAlpha, hatlamSq, hatnu, invSigAlpha0, hatTauSq, hatSigmaSq, hatZeta, a, b, progress) {
    .Call(`_pqrBayes_Bhorse`, xx, y, W, maxSteps, hatBeta, hatAlpha, hatlamSq, hatnu, invSigAlpha0, hatTauSq, hatSigmaSq, hatZeta, a, b, progress)
}

Bhorse_reg <- function(xx, y, W, maxSteps, hatBeta, hatAlpha, hatlamSq, hatnu, invSigAlpha0, hatTauSq, hatcSq, hatSigmaSq, hatZeta, a, b, c, d, progress) {
    .Call(`_pqrBayes_Bhorse_reg`, xx, y, W, maxSteps, hatBeta, hatAlpha, hatlamSq, hatnu, invSigAlpha0, hatTauSq, hatcSq, hatSigmaSq, hatZeta, a, b, c, d, progress)
}

#' fit Bayesian penalized quantile regression for a sparse linear model, binary LASSO, group LASSO, or varying coefficient model based on spike-and-slab priors and/or the horseshoe family of priors
#' 
#' @keywords models
#' @param g the matrix of predictors (subject to selection). Users do not need to specify an intercept which will be automatically included. 
#' @param y the response variable.
#' @param u a vector of effect modifying variable of the quantile varying coefficient model. When fitting a linear model or group LASSO, u = NULL. The default value is NULL.
#' @param e a matrix of clinical covariates not subject to selection. The default value is NULL.
#' @param quant the quantile level specified by users. Required when robust = TRUE. Ignored (set to NULL) when robust = FALSE.The default value is 0.5.
#' @param d a positive integer denotes the group size. When fitting a sparse linear or varying coefficient model, d = NULL. The default value is NULL.
#' @param iterations the number of MCMC iterations. The default value is 10,000.
#' @param burn.in the number of burn-in iterations. If NULL, the first half of MCMC iterations will be discarded as burn-ins.
#' @param robust logical flag. If TRUE, robust methods are used. Otherwise, non-robust methods are used. The default value is TRUE.
#' @param prior the type of prior used for variable selection. Users can specify "SS" for the spike-and-slab prior, "HS" for the horseshoe prior, "HS+" for the horseshoe plus prior, "RHS" for the regularized horseshoe prior and "Laplace" for the Laplace prior. The default value is "SS".
#' @param model the model to be fitted. Users can specify "linear" for a sparse linear model (with a continuous response), "binary" for binary LASSO, "group" for group LASSO and "VC" for a varying coefficient model.
#' @param hyper a named list of hyper-parameters. The default value is NULL.
#' @param debugging logical flag. If TRUE, progress will be output to the console and extra information will be returned. The default value is FALSE.
#'
#' @details 
#' The sparse linear quantile regression model described in "\code{\link{data}}" is:
#' \deqn{Y_{i}=\sum_{k=1}^{q} E_{ik} \beta_k +\sum_{j=0}^{p}X_{ij}\gamma_j +\epsilon_{i},}
#' where \eqn{\beta_k}'s are the regression coefficients for clinical covariates and \eqn{\gamma_j}'s are the regression coefficients of \eqn{\boldsymbol X}.
#' 
#' The binary quantile regression model described in "\code{\link{data}}" is:
#' \deqn{\tilde{Y}_{i}=\sum_{k=1}^{q} E_{ik} \beta_k +\sum_{j=0}^{p}X_{ij}\gamma_j +\epsilon_{i},}
#' where \eqn{\beta_k}'s are the regression coefficients for clinical covariates and \eqn{\gamma_j}'s are the regression coefficients of \eqn{\boldsymbol X}.
#' 
#' \eqn{Y_{i}=1} if \eqn{\tilde{Y}_{i}>0} and \eqn{Y_{i}=0} otherwise.
#' 
#' The group LASSO model described in "\code{\link{data}}" is:
#' \deqn{Y_{i}=\sum_{k=1}^{q} E_{ik} \beta_k + X_{i0}\gamma_0+ \sum_{j=1}^{m}\boldsymbol{X_{ij}^\top}\boldsymbol{\gamma_j} +\epsilon_{i},}
#' where \eqn{\beta_k}'s are the regression coefficients for clinical covariates and \eqn{\boldsymbol{\gamma_j} = (\gamma_{j1},\dots,\gamma_{jd})^\top} is the vector of regression coefficients of the \eqn{n \times d} matrix \eqn{\boldsymbol X_{j}}.
#' 
#' The quantile varying coefficient model described in "\code{\link{data}}" is:
#' \deqn{Y_{i}=\sum_{k=1}^{q} E_{ik} \beta_k +\sum_{j=0}^{p}\gamma_j(V_i)X_{ij} +\epsilon_{i},}
#' where \eqn{\beta_k}'s are the regression coefficients for the clinical covariates, and \eqn{\gamma_j}'s are the varying intercept and varying coefficients for predictors (e.g. genetic factors), respectively.
#'
#' 
#' Users can modify the hyper-parameters by providing a named list of hyper-parameters via the argument `hyper'.
#' The list can have the following named components
#' \describe{
#'   \item{a0, b0: }{ shape parameters of the Beta priors (\eqn{\pi^{a_{0}-1}(1-\pi)^{b_{0}-1}}) on \eqn{\pi_{0}}.}
#'   \item{c1, c2: }{ the shape parameter and the rate parameter of the Gamma prior on \eqn{\nu}.}
#' }
#' 
#' Please check the references for more details about the prior distributions.
#' 
#' @return an object of class "pqrBayes" is returned, which is a list with components:
#' \item{obj}{a list of posterior samples from the MCMC and other parameters}
#' \item{coefficients}{a list of posterior estimates of coefficients}
#' 
#' @examples
#' ## The Bayesian regularized quantile regression under linear and binary model
#' data(data)
#' data = data$data_linear
#' g=data$g
#' y=data$y
#' e=data$e
#' data(data)
#' data_1=data$data_binary
#' g_1=data_1$g
#' y_1=data_1$y
#' e_1=data_1$e
#' 
#' fit1=pqrBayes(g,y,e,model="linear")
#' fit1_1=pqrBayes(g_1,y_1,e_1,model="binary")

#' \donttest{
#'
#' ## Non-sparse robust example (Bayesian Quantile LASSO)
#' fit2 <- pqrBayes(g, y, e, prior = "Laplace", model = "linear")
#'
#' ## Non-robust example (Bayesian LASSO)
#' fit3 <- pqrBayes(g, y, e, quant = NULL, robust = FALSE, model = "linear")
#' }

#' 
#' ## The Bayesian quantile group LASSO model
#' data(data)
#' data = data$data_group
#' g=data$g
#' y=data$y
#' e=data$e
#' 
#' fit1=pqrBayes(g,y,e,d=3,model="group")
#' \donttest{
#'
#' ## Non-sparse robust Bayesian version
#' fit2 <- pqrBayes(g, y, e, d = 3, prior = "Laplace", model = "group")
#'
#' ## Non-robust Bayesian version
#' fit3 <- pqrBayes(g, y, e, d = 3, quant = NULL, robust = FALSE, model = "group")
#' }

#' ## The regularized Bayesian quantile varying coefficient model
#' data(data)
#' data = data$data_varying
#' g=data$g
#' y=data$y
#' e=data$e
#' u=data$u
#' fit1=pqrBayes(g,y,u,e,model="VC")
#'
#' \donttest{
#'
#' ## Non-sparse robust Bayesian example
#' fit2 <- pqrBayes(g, y, u, e, prior = "Laplace", model = "VC")
#'
#' ## Non-robust Bayesian example
#' fit3 <- pqrBayes(g, y, u, e, quant = NULL,robust = FALSE, model = "VC")
#' }

#' @export
#' @references
#' 
#' Fan, K., Subedi, S., Yang, G., Lu, X., Ren, J. and Wu, C. (2024). Is Seeing Believing? A Practitioner's Perspective on High-dimensional Statistical Inference in Cancer Genomics Studies. {\emph{Entropy}, 26(9).794} \doi{10.3390/e26090794}
#' 
#' Zhou, F., Ren, J., Ma, S. and Wu, C. (2023). The Bayesian regularized quantile varying coefficient model.
#'  {\emph{Computational Statistics & Data Analysis}, 187, 107808} \doi{10.1016/j.csda.2023.107808}
#'  
#' Ren, J., Zhou, F., Li, X., Ma, S., Jiang, Y. and Wu, C. (2023). Robust Bayesian variable selection for gene-environment interactions. 
#' {\emph{Biometrics}, 79(2), 684-694} \doi{10.1111/biom.13670}
#' 
#' Fan, K. and Wu, C. (2025). A New Robust Binary Bayesian LASSO. 
#' {\emph{Stat}, 14 (3), e70078} \doi{10.1002/sta4.70078}
#' 
#' Fan, K., Srijana, S., Dissanayake, V. and Wu, C. (2026). Robust Bayesian high-dimensional variable selection and inference with the horseshoe family of priors. 
#' {\emph{Computational Statistics & Data Analysis}, 219, 108358} \doi{10.1016/j.csda.2026.108358}
#'
#' Ren, J., Zhou, F., Li, X., Chen, Q., Zhang, H., Ma, S., Jiang, Y. and Wu, C. (2020) Semi-parametric Bayesian variable selection for gene-environment interactions.
#' {\emph{Statistics in Medicine}, 39: 617– 638} \doi{10.1002/sim.8434}




pqrBayes <- function(g, y, u=NULL, e=NULL, d = NULL, quant = 0.5, iterations = 10000, 
                     burn.in = NULL, robust = TRUE, prior = "SS", 
                     model,
                     hyper = NULL, debugging = FALSE) {
  
  if (prior %in% c("HS", "HS+", "RHS") && model != "linear") {
    stop("Priors 'HS', 'HS+' and 'RHS' are currently supported only when model = 'linear'.")
  }
  if (!prior %in% c("SS", "HS", "HS+", "RHS", "Laplace")) {
    stop("prior should be one of: 'SS', 'HS', 'HS+', 'RHS' or 'Laplace'.")
  }
  # quant check
  if (robust) {
    if (is.null(quant)) {
      stop("quant must be specified when robust = TRUE.")
    }
  } else {
    if (!is.null(quant)) {
      stop("quant must be NULL when robust = FALSE.")
    }
  }
  
  if (model != "VC" && !is.null(u)) {
    
    stop("Argument 'u' should be NULL unless model = 'VC'.")
    
  }
  
  if (model == "VC" && is.null(u)) {
    
    stop("Argument 'u' must be provided when model = 'VC'.")
    
  }
  
  
  if (model != "group" && !is.null(d)) {
    
    stop("Argument 'd' should be NULL unless model = 'group'.")
    
  }
  
  if (model == "group" && is.null(d)) {
    
    stop("Argument 'd' must be provided when model = 'group'.")
    
  }
  
  if (model == "VC") {
    
    fit = pqrBayes_vc(g, y, u, e, quant, iterations, burn.in, robust, prior, hyper, debugging)
    
  } else if (model == "linear") {
    fit = pqrBayes_lin(g, y, e, quant, iterations, burn.in, robust, prior, hyper, debugging)
    
  } else if (model == "binary") {
    fit = pqrBayes_bin(g, y, e, quant, iterations, burn.in, robust, prior, hyper, debugging)
    
  }else if (model == "group") {
    fit = pqrBayes_g(g, y, e, d, quant, iterations, burn.in, robust, prior, hyper, debugging)
    
  } else {
    stop("model should be one of: 'VC', 'linear', 'binary' or 'group'.")
  }
  
  return(fit)
}

#' @useDynLib pqrBayes, .registration = TRUE
#' @importFrom Rcpp sourceCpp
NULL

"_PACKAGE"
#' @keywords overview
#' @name pqrBayes-package
#' @title Bayesian penalized quantile regression for the sparse linear model, binary LASSO, group LASSO and varying coefficient models based on spike-and-slab priors and/or the horseshoe family of priors
#' @aliases pqrBayes-package
#' @description In this package, we implement Bayesian penalized quantile regression for a sparse linear model (with a continuous response), binary LASSO, group LASSO and quantile varying coefficient (VC) models. 
#' The point-mass spike-and-slab priors and horseshoe family of (horseshoe, horseshoe+ and regularized horseshoe) priors have been incorporated in the Bayesian hierarchical
#' models to facilitate Bayesian shrinkage estimation, variable selection and statistical inference. For the spike-and-slab priors, the four default methods are Bayesian
#' regularized quantile regression with spike-and-slab priors under the sparse linear (i.e. LASSO), binary LASSO, group LASSO and VC model, correspondingly. In addition to the default methods, 
#' users can also choose models without robustness and/or spike--and--slab priors. Furthermore, under sparse linear models, we have implemented the horseshoe family of (horseshoe, horseshoe+
#' and regularized horseshoe) priors and the three non-robust alternatives. Currently, the horseshoe family of priors is only implemented under the sparse linear model (with a continuous response).
#' 
#' @details The user friendly, integrated interface \strong{pqrBayes()} allows users to flexibly choose fitting models by specifying the following parameters:
#' \tabular{rl}{
#' robust: \tab whether to fit a robust sparse quantile regression model (the sparse linear model \cr\tab with a continuous response, binary LASSO, group LASSO or varying coefficient models) \cr\tab or their non-robust counterparts. \cr\cr
#' prior: \tab specify which prior to use (the spike-and-slab prior, Laplace prior \cr\tab and the horseshoe family of priors). \cr\cr
#' model: \tab whether to fit a sparse linear model (with a continuous response), binary LASSO, group LASSO \cr\tab or a varying coefficient model.
#' }
#'
#' The function pqrBayes() returns a pqrBayes object that stores the posterior estimates of regression coefficients.
#'
#' @references
#' Fan, K., Subedi, S., Yang, G., Lu, X., Ren, J. and Wu, C. (2024). Is Seeing Believing? A Practitioner's Perspective on High-dimensional Statistical Inference in Cancer Genomics Studies. {\emph{Entropy}, 26(9).794} \doi{10.3390/e26090794}  
#'
#' Zhou, F., Ren, J., Ma, S. and Wu, C. (2023). The Bayesian regularized quantile varying coefficient model.
#'  {\emph{Computational Statistics & Data Analysis}, 187, 107808} \doi{10.1016/j.csda.2023.107808}
#'  
#' Ren, J., Zhou, F., Li, X., Ma, S., Jiang, Y. and Wu, C. (2023). Robust Bayesian variable selection for gene-environment interactions. 
#' {\emph{Biometrics}, 79(2), 684-694} \doi{10.1111/biom.13670}
#'
#' Fan, K. and Wu, C. (2025). A New Robust Binary Bayesian LASSO. 
#' {\emph{Stat}, 14 (3), e70078} \doi{10.1002/sta4.70078}
#' 
#' Fan, K., Srijana, S., Dissanayake, V. and Wu, C. (2026). Robust Bayesian high-dimensional variable selection and inference with the horseshoe family of priors 
#' {\emph{Computational Statistics & Data Analysis}, 219, 108358}  \doi{10.1016/j.csda.2026.108358}
#' 
#' Wu, C., and Ma, S. (2015). A selective review of robust variable selection with applications in bioinformatics.
#' {\emph{Briefings in Bioinformatics}, 16(5), 873–883} \doi{10.1093/bib/bbu046}
#'
#' Zhou, F., Ren, J., Lu, X., Ma, S. and Wu, C. (2021). Gene–Environment Interaction: a Variable Selection Perspective. 
#' {\emph{Epistasis. Methods in Molecular Biology.} 2212:191–223} \url{https://link.springer.com/protocol/10.1007/978-1-0716-0947-7_13}
#'
#' Ren, J., Zhou, F., Li, X., Chen, Q., Zhang, H., Ma, S., Jiang, Y. and Wu, C. (2020) Semi-parametric Bayesian variable selection for gene-environment interactions.
#' {\emph{Statistics in Medicine}, 39: 617– 638} \doi{10.1002/sim.8434}
#'
#' Ren, J., Zhou, F., Li, X., Wu, C. and Jiang, Y. (2019) spinBayes: Semi-Parametric Gene-Environment Interaction via Bayesian Variable Selection.
#' R package version 0.1.0. \url{https://CRAN.R-project.org/package=spinBayes}
#'
#' Wu, C., Jiang, Y., Ren, J., Cui, Y. and Ma, S. (2018). Dissecting gene-environment interactions: A penalized robust approach accounting for hierarchical structures.
#' {\emph{Statistics in Medicine}, 37:437–456} \doi{10.1002/sim.7518}
#'
#' Wu, C., Shi, X., Cui, Y. and Ma, S. (2015). A penalized robust semiparametric approach for gene-environment interactions.
#' {\emph{Statistics in Medicine}, 34 (30): 4016–4030} \doi{10.1002/sim.6609}
#'
#' Wu, C., Cui, Y., and Ma, S. (2014). Integrative analysis of gene–environment interactions under a multi–response partially linear varying coefficient model.
#' {\emph{Statistics in Medicine}, 33(28), 4988–4998} \doi{10.1002/sim.6287}
#'
#' Wu, C., Zhong, P.S. and Cui, Y. (2018). Additive varying–coefficient model for nonlinear gene–environment interactions.
#' {\emph{Statistical Applications in Genetics and Molecular Biology}, 17(2)} \doi{10.1515/sagmb-2017-0008}
#'
#' Wu, C., Zhong, P.S. and Cui, Y. (2013). High dimensional variable selection for gene-environment interactions.
#' {\emph{Technical Report. Michigan State University.}}
#'
#' @seealso \code{\link{pqrBayes}}
NULL

NonRobust_bin <- function(g, y, e,iterations,prior,debugging){
  
  p = dim(g)[2]
  
  x = cbind(1,g)
  n = length(y); 
  
  if(!is.null(e)){
    q = dim(e)[2]
    hatAlpha=rep(1,(1+q))
    invSigAlpha0 = diag(10^-3, (1+q))
    w = cbind(1,e)
  }else{
    q = 0
    hatAlpha=rep(1,1)
    invSigAlpha0 = diag(10^-3, 1)
    w = matrix(1,n,1)
  }
  ystar = rep(1,n)
  hatBeta = rep(1,p)
  hatInvTauSq = rep(1,p)
  hatLambdaSq=1
 
  aStar=1 
  bStar=1
  alpha=1 
  gamma=1
  hatPi=0.5
  sh1=1
  sh0=1
  progress=0
  
  progress = ifelse(debugging, 10^(floor(log10(iterations))-1), 0)
  if(prior == "SS"){
    fit=BBLSS(g, y, ystar, w, iterations, hatAlpha, hatBeta, hatInvTauSq, invSigAlpha0, hatPi, hatLambdaSq, aStar, bStar, alpha, gamma, sh1, sh0, progress)}
  else if (prior == "Laplace"){fit=BBL(g, y, ystar, w, iterations, hatBeta, hatAlpha, hatInvTauSq, invSigAlpha0, hatLambdaSq, aStar, bStar, alpha, gamma, progress)}
  else{stop("The specified prior is currently not supported.")}
  out=list(fit=fit,iterations=iterations)
  return(out)
}

#' print a pqrBayes result
#'
#' Print a pqrBayes result
#'
#' @param x pqrBayes result.
#' @param digits significant digits in printout.
#' @param ... other print arguments.
#' @usage \method{print}{pqrBayes}(x, digits = max(3, getOption("digits") - 3), \dots)
#' @return No return value, called for side effects.
#' @seealso \code{\link{pqrBayes}}
#' @export
print.pqrBayes=function(x, digits = max(3, getOption("digits") - 3),...){
  print(x, digits)
}


#' print a pqrBayes.pred object
#'
#' Print a summary of a pqrBayes.pred object
#'
#' @param x pqrBayes.pred object.
#' @param digits significant digits in printout.
#' @param ... other print arguments
#' @usage \method{print}{pqrBayes.pred}(x, digits = max(3, getOption("digits") - 3), \dots)
#' @return No return value, called for side effects.
#' @seealso \code{\link{predict_pqrBayes}}
#' @export
print.pqrBayes.pred=function(x, digits = max(3, getOption("digits") - 3),...){
  cat("\ncheck loss:\n")
  print(x$error, digits)
  cat("\npredicted:\n ")
  print(x$y.pred,digits)
}


#' print a select.pqrBayes object
#'
#' Print a summary of a select.pqrBayes object
#'
#' @param x pqrBayes.select object.
#' @param digits significant digits in printout.
#' @param ... other print arguments
#' @usage \method{print}{pqrBayes.select}(x, digits = max(3, getOption("digits") - 3), \dots)
#' @return No return value, called for side effects.
#' @seealso \code{\link{pqrBayes.select}}
#' @export
print.pqrBayes.select=function(x, digits = max(3, getOption("digits") - 3),...){
  cat("\nMethod:\n")
  print(x$method)
  cat("\n")
  print(x$id)
}

#' 95\% empirical coverage probability for a pqrBayes object
#'
#'
#'@description
#'Calculate 95\% empirical coverage probabilities for regression coefficients under a sparse linear model (with a continuous response), binary LASSO, group LASSO and VC models, respectively.
#'
#' @param object the pqrBayes object.
#' @param coefficient the vector of true regression coefficients under the sparse linear model (with a continuous response), binary LASSO, group LASSO, or the matrix of true varying coefficients evaluated on the grid points under a varying coefficient model.
#' @param u.grid the vector of grid points under a varying coefficient model. When assessing empirical coverage probabilities under a sparse linear model, binary LASSO or group LASSO, u.grid = NULL.
#' @param model the model to be fitted. Users can also choose "linear" for a sparse linear model (with a continuous response), "binary" for binary LASSO, "group" for group LASSO, and "VC" for a sparse varying coefficient model.
#' @usage coverage(object,coefficient,u.grid=NULL,model="linear")
#' @return c
#' @seealso \code{\link{pqrBayes}}
#' @examples
#' ## The Bayesian regularized quantile regression model
#' data(data)
#' data = data$data_linear
#' g=data$g
#' y=data$y
#' e=data$e
#' coeff = data$coeff
#' fit1=pqrBayes(g,y,e,d = NULL,quant=0.5,model="linear")
#' coverage=coverage(fit1,coeff,model="linear")
#' @export
coverage = function(object,coefficient,u.grid=NULL,model="linear"){
  if(model=="VC"){
    c = coverage_vc(object,coefficient,u.grid)
  }
  else if(model=="linear"){
    c = coverage_lin(object,coefficient,u.grid=NULL)
  }
  else if(model=="binary"){
    c = coverage_lin(object,coefficient,u.grid=NULL)
  }
  else if(model=="group"){
    c = coverage_lin(object,coefficient,u.grid=NULL)
  }
  else{
    stop("model should be either VC, linear, binary or group")
  }
  return(c)
}

predict_lin_robust=function(obj, g.new, e.new, y.new, quant,...){
  p = dim(g.new)[2]
  x = cbind(1,g.new)
  n = dim(g.new)[1]; 
  coeff.est = c()
  for(j in 1:dim(obj$coefficients$GS.beta)[2])
  {
    t1=as.matrix(obj$coefficients$GS.beta[,j])
    coeff.est[j] = stats::median(t1)
  }
  
  if(!is.null(e.new)){
    q = dim(e.new)[2]
    
    beta.hat=rep(0,q+1)
    for (i in 1:(q+1)) {
      t2=as.matrix(obj$coefficients$GS.alpha[,i])
      beta.hat[i] = stats::median(t2)
    }
    gamma.hat = c(beta.hat[1],coeff.est)
    
    # prediction error
    y.hat=res=rep(0,n)
    for (i in 1:n) {
      y.hat[i]=t(x[i,])%*%gamma.hat+t(e.new[i,])%*%beta.hat[-1]
      if((y.new[i]-y.hat[i]) >= 0){res[i] = quant*(y.new[i]-y.hat[i])}
      else{res[i] = (quant -1)*(y.new[i]-y.hat[i])}
    }
    
    pred.error=mean(res)
    
    
  }else{
    
    t2=as.matrix(obj$coefficients$GS.alpha)
    beta.hat = stats::median(t2)
    
    gamma.hat = c(beta.hat,coeff.est)
    
    # prediction error
    y.hat=res=rep(0,n)
    for (i in 1:n) {
      y.hat[i]=t(x[i,])%*%gamma.hat
      if((y.new[i]-y.hat[i]) >= 0){res[i] = quant*(y.new[i]-y.hat[i])}
      else{res[i] = (quant -1)*(y.new[i]-y.hat[i])}
    }
    
    pred.error=mean(res)
    
  }
  
  
  pqrBayes_lin_robust= list(error=pred.error, y.pred=y.hat)
  #class(pqrBayes.pred) = "pqrBayes.pred"
  return(pqrBayes_lin_robust)
  #pred
}

#' Estimation and estimation accuracy for a pqrBayes object
#'
#' Calculate estimated regression coefficients with estimation accuracy from a sparse linear model (with a continuous response), binary LASSO, group LASSO and quantile VC models, respectively.
#'
#' @param object an object of class `pqrBayes'.
#' @param coefficient the vector of quantile regression coefficients under a sparse linear model (with a continuous response), binary LASSO and group LASSO or the matrix of true varying coefficients evaluated on the grid points under a varying coefficient model.
#' @param u.grid the vector of grid points under a varying coefficient model. When fitting a sparse linear model, binary LASSO or group LASSO, u.grid = NULL. The default value is NULL.
#' @param model the model to be fitted. Users can choose "linear" for a sparse linear model (with a continuous response), "binary" for binary LASSO, "group" for group LASSO or "VC" for a varying coefficient model.
#' @usage estimation.pqrBayes(object,coefficient,u.grid=NULL,model)
#' @return  an object of class `pqrBayes.est' is returned, which is a list with components:
#' \item{error}{mean square error or integrated mean square errors and total integrated mean square error.}
#' \item{coeff.est}{estimated values of the regression coefficients or the varying coefficients.}
#' @seealso \code{\link{pqrBayes}}
#' @examples
#' ## The quantile regression model
#' data(data)
#' data = data$data_linear
#' g=data$g
#' y=data$y
#' e=data$e
#' coeff = data$coeff
#' fit1=pqrBayes(g,y,e,model="linear")
#' estimation=estimation.pqrBayes(fit1,coeff,model="linear")

#'
#' @export
estimation.pqrBayes = function(object,coefficient,u.grid=NULL,model){
  if (model != "VC" && !is.null(u.grid)) {
    
    stop("Argument 'u.grid' should be NULL unless model = 'VC'.")
    
  }
  
  if (model == "VC" && is.null(u.grid)) {
    
    stop("Argument 'u.grid' must be provided when model = 'VC'.")
    
  }
  if(model=="VC"){
    pqrBayes.est = estimation_vc(object,coefficient,u.grid)
  }
  else if(model=="linear"){
    pqrBayes.est = estimation_lin(object,coefficient)
  }
  else if(model=="binary"){
    pqrBayes.est = estimation_lin(object,coefficient)
  }
  else if(model=="group"){
    pqrBayes.est = estimation_lin(object,coefficient)
  }
  else{
    stop("model should be either VC, linear,binary or group")
  }
  class(pqrBayes.est) = "pqrBayes.est"
  return(pqrBayes.est)
}
  

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pqrBayes-package.R
\name{pqrBayes-package}
\alias{pqrBayes-package}
\title{Bayesian penalized quantile regression for the sparse linear model, binary LASSO, group LASSO and varying coefficient models based on spike-and-slab priors and/or the horseshoe family of priors}
\description{
In this package, we implement Bayesian penalized quantile regression for a sparse linear model (with a continuous response), binary LASSO, group LASSO and quantile varying coefficient (VC) models. 
The point-mass spike-and-slab priors and horseshoe family of (horseshoe, horseshoe+ and regularized horseshoe) priors have been incorporated in the Bayesian hierarchical
models to facilitate Bayesian shrinkage estimation, variable selection and statistical inference. For the spike-and-slab priors, the four default methods are Bayesian
regularized quantile regression with spike-and-slab priors under the sparse linear (i.e. LASSO), binary LASSO, group LASSO and VC model, correspondingly. In addition to the default methods, 
users can also choose models without robustness and/or spike--and--slab priors. Furthermore, under sparse linear models, we have implemented the horseshoe family of (horseshoe, horseshoe+
and regularized horseshoe) priors and the three non-robust alternatives. Currently, the horseshoe family of priors is only implemented under the sparse linear model (with a continuous response).
}
\details{
The user friendly, integrated interface \strong{pqrBayes()} allows users to flexibly choose fitting models by specifying the following parameters:
\tabular{rl}{
robust: \tab whether to fit a robust sparse quantile regression model (the sparse linear model \cr\tab with a continuous response, binary LASSO, group LASSO or varying coefficient models) \cr\tab or their non-robust counterparts. \cr\cr
prior: \tab specify which prior to use (the spike-and-slab prior, Laplace prior \cr\tab and the horseshoe family of priors). \cr\cr
model: \tab whether to fit a sparse linear model (with a continuous response), binary LASSO, group LASSO \cr\tab or a varying coefficient model.
}

The function pqrBayes() returns a pqrBayes object that stores the posterior estimates of regression coefficients.
}
\references{
Fan, K., Subedi, S., Yang, G., Lu, X., Ren, J. and Wu, C. (2024). Is Seeing Believing? A Practitioner's Perspective on High-dimensional Statistical Inference in Cancer Genomics Studies. {\emph{Entropy}, 26(9).794} \doi{10.3390/e26090794}  

Zhou, F., Ren, J., Ma, S. and Wu, C. (2023). The Bayesian regularized quantile varying coefficient model.
 {\emph{Computational Statistics & Data Analysis}, 187, 107808} \doi{10.1016/j.csda.2023.107808}
 
Ren, J., Zhou, F., Li, X., Ma, S., Jiang, Y. and Wu, C. (2023). Robust Bayesian variable selection for gene-environment interactions. 
{\emph{Biometrics}, 79(2), 684-694} \doi{10.1111/biom.13670}

Fan, K. and Wu, C. (2025). A New Robust Binary Bayesian LASSO. 
{\emph{Stat}, 14 (3), e70078} \doi{10.1002/sta4.70078}

Fan, K., Srijana, S., Dissanayake, V. and Wu, C. (2026). Robust Bayesian high-dimensional variable selection and inference with the horseshoe family of priors 
{\emph{Computational Statistics & Data Analysis}, 219, 108358}  \doi{10.1016/j.csda.2026.108358}

Wu, C., and Ma, S. (2015). A selective review of robust variable selection with applications in bioinformatics.
{\emph{Briefings in Bioinformatics}, 16(5), 873–883} \doi{10.1093/bib/bbu046}

Zhou, F., Ren, J., Lu, X., Ma, S. and Wu, C. (2021). Gene–Environment Interaction: a Variable Selection Perspective. 
{\emph{Epistasis. Methods in Molecular Biology.} 2212:191–223} \url{https://link.springer.com/protocol/10.1007/978-1-0716-0947-7_13}

Ren, J., Zhou, F., Li, X., Chen, Q., Zhang, H., Ma, S., Jiang, Y. and Wu, C. (2020) Semi-parametric Bayesian variable selection for gene-environment interactions.
{\emph{Statistics in Medicine}, 39: 617– 638} \doi{10.1002/sim.8434}

Ren, J., Zhou, F., Li, X., Wu, C. and Jiang, Y. (2019) spinBayes: Semi-Parametric Gene-Environment Interaction via Bayesian Variable Selection.
R package version 0.1.0. \url{https://CRAN.R-project.org/package=spinBayes}

Wu, C., Jiang, Y., Ren, J., Cui, Y. and Ma, S. (2018). Dissecting gene-environment interactions: A penalized robust approach accounting for hierarchical structures.
{\emph{Statistics in Medicine}, 37:437–456} \doi{10.1002/sim.7518}

Wu, C., Shi, X., Cui, Y. and Ma, S. (2015). A penalized robust semiparametric approach for gene-environment interactions.
{\emph{Statistics in Medicine}, 34 (30): 4016–4030} \doi{10.1002/sim.6609}

Wu, C., Cui, Y., and Ma, S. (2014). Integrative analysis of gene–environment interactions under a multi–response partially linear varying coefficient model.
{\emph{Statistics in Medicine}, 33(28), 4988–4998} \doi{10.1002/sim.6287}

Wu, C., Zhong, P.S. and Cui, Y. (2018). Additive varying–coefficient model for nonlinear gene–environment interactions.
{\emph{Statistical Applications in Genetics and Molecular Biology}, 17(2)} \doi{10.1515/sagmb-2017-0008}

Wu, C., Zhong, P.S. and Cui, Y. (2013). High dimensional variable selection for gene-environment interactions.
{\emph{Technical Report. Michigan State University.}}
}
\seealso{
\code{\link{pqrBayes}}
}
\keyword{overview}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coverage.R
\name{coverage}
\alias{coverage}
\title{95\% empirical coverage probability for a pqrBayes object}
\usage{
coverage(object,coefficient,u.grid=NULL,model)
}
\arguments{
\item{object}{the pqrBayes object.}

\item{coefficient}{the vector of true regression coefficients under the sparse linear model (with a continuous response), binary LASSO, group LASSO, or the matrix of true varying coefficients evaluated on the grid points under a varying coefficient model.}

\item{u.grid}{the vector of grid points under a varying coefficient model. When assessing empirical coverage probabilities under a sparse linear model, binary LASSO or group LASSO, u.grid = NULL. The default value is NULL.}

\item{model}{the model to be fitted. Users can also choose "linear" for a sparse linear model (with a continuous response), "binary" for binary LASSO, "group" for group LASSO, and "VC" for a sparse varying coefficient model.}
}
\value{
c
}
\description{
Calculate 95\% empirical coverage probabilities for regression coefficients under a sparse linear model (with a continuous response), binary LASSO, group LASSO and VC models, respectively.
}
\examples{
## The Bayesian regularized quantile regression model
data(data)
data = data$data_linear
g=data$g
y=data$y
e=data$e
coeff = data$coeff
fit1=pqrBayes(g,y,e,model="linear")
coverage=coverage(fit1,coeff,model="linear")
}
\seealso{
\code{\link{pqrBayes}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pqrBayes.R
\name{pqrBayes}
\alias{pqrBayes}
\title{fit Bayesian penalized quantile regression for a sparse linear model, binary LASSO, group LASSO, or varying coefficient model based on spike-and-slab priors and/or the horseshoe family of priors}
\usage{
pqrBayes(
  g,
  y,
  u = NULL,
  e = NULL,
  d = NULL,
  quant = 0.5,
  iterations = 10000,
  burn.in = NULL,
  robust = TRUE,
  prior = "SS",
  model,
  hyper = NULL,
  debugging = FALSE
)
}
\arguments{
\item{g}{the matrix of predictors (subject to selection). Users do not need to specify an intercept which will be automatically included.}

\item{y}{the response variable.}

\item{u}{a vector of effect modifying variable of the quantile varying coefficient model. When fitting a linear model or group LASSO, u = NULL. The default value is NULL.}

\item{e}{a matrix of clinical covariates not subject to selection. The default value is NULL.}

\item{d}{a positive integer denotes the group size. When fitting a sparse linear or varying coefficient model, d = NULL. The default value is NULL.}

\item{quant}{the quantile level specified by users. Required when robust = TRUE. Ignored (set to NULL) when robust = FALSE.The default value is 0.5.}

\item{iterations}{the number of MCMC iterations. The default value is 10,000.}

\item{burn.in}{the number of burn-in iterations. If NULL, the first half of MCMC iterations will be discarded as burn-ins.}

\item{robust}{logical flag. If TRUE, robust methods are used. Otherwise, non-robust methods are used. The default value is TRUE.}

\item{prior}{the type of prior used for variable selection. Users can specify "SS" for the spike-and-slab prior, "HS" for the horseshoe prior, "HS+" for the horseshoe plus prior, "RHS" for the regularized horseshoe prior and "Laplace" for the Laplace prior. The default value is "SS".}

\item{model}{the model to be fitted. Users can specify "linear" for a sparse linear model (with a continuous response), "binary" for binary LASSO, "group" for group LASSO and "VC" for a varying coefficient model.}

\item{hyper}{a named list of hyper-parameters. The default value is NULL.}

\item{debugging}{logical flag. If TRUE, progress will be output to the console and extra information will be returned. The default value is FALSE.}
}
\value{
an object of class "pqrBayes" is returned, which is a list with components:
\item{obj}{a list of posterior samples from the MCMC and other parameters}
\item{coefficients}{a list of posterior estimates of coefficients}
}
\description{
fit Bayesian penalized quantile regression for a sparse linear model, binary LASSO, group LASSO, or varying coefficient model based on spike-and-slab priors and/or the horseshoe family of priors
}
\details{
The sparse linear quantile regression model described in "\code{\link{data}}" is:
\deqn{Y_{i}=\sum_{k=1}^{q} E_{ik} \beta_k +\sum_{j=0}^{p}X_{ij}\gamma_j +\epsilon_{i},}
where \eqn{\beta_k}'s are the regression coefficients for clinical covariates and \eqn{\gamma_j}'s are the regression coefficients of \eqn{\boldsymbol X}.

The binary quantile regression model described in "\code{\link{data}}" is:
\deqn{\tilde{Y}_{i}=\sum_{k=1}^{q} E_{ik} \beta_k +\sum_{j=0}^{p}X_{ij}\gamma_j +\epsilon_{i},}
where \eqn{\beta_k}'s are the regression coefficients for clinical covariates and \eqn{\gamma_j}'s are the regression coefficients of \eqn{\boldsymbol X}.

\eqn{Y_{i}=1} if \eqn{\tilde{Y}_{i}>0} and \eqn{Y_{i}=0} otherwise.

The group LASSO model described in "\code{\link{data}}" is:
\deqn{Y_{i}=\sum_{k=1}^{q} E_{ik} \beta_k + X_{i0}\gamma_0+ \sum_{j=1}^{m}\boldsymbol{X_{ij}^\top}\boldsymbol{\gamma_j} +\epsilon_{i},}
where \eqn{\beta_k}'s are the regression coefficients for clinical covariates and \eqn{\boldsymbol{\gamma_j} = (\gamma_{j1},\dots,\gamma_{jd})^\top} is the vector of regression coefficients of the \eqn{n \times d} matrix \eqn{\boldsymbol X_{j}}.

The quantile varying coefficient model described in "\code{\link{data}}" is:
\deqn{Y_{i}=\sum_{k=1}^{q} E_{ik} \beta_k +\sum_{j=0}^{p}\gamma_j(V_i)X_{ij} +\epsilon_{i},}
where \eqn{\beta_k}'s are the regression coefficients for the clinical covariates, and \eqn{\gamma_j}'s are the varying intercept and varying coefficients for predictors (e.g. genetic factors), respectively.


Users can modify the hyper-parameters by providing a named list of hyper-parameters via the argument `hyper'.
The list can have the following named components
\describe{
  \item{a0, b0: }{ shape parameters of the Beta priors (\eqn{\pi^{a_{0}-1}(1-\pi)^{b_{0}-1}}) on \eqn{\pi_{0}}.}
  \item{c1, c2: }{ the shape parameter and the rate parameter of the Gamma prior on \eqn{\nu}.}
}

Please check the references for more details about the prior distributions.
}
\examples{
## The Bayesian regularized quantile regression under linear and binary model
data(data)
data = data$data_linear
g=data$g
y=data$y
e=data$e
data(data)
data_1=data$data_binary
g_1=data_1$g
y_1=data_1$y
e_1=data_1$e

fit1=pqrBayes(g,y,e,model="linear")
fit1_1=pqrBayes(g_1,y_1,e_1,model="binary")
\donttest{

## Non-sparse robust example (Bayesian Quantile LASSO)
fit2 <- pqrBayes(g, y, e, prior = "Laplace", model = "linear")

## Non-robust example (Bayesian LASSO)
fit3 <- pqrBayes(g, y, e, quant = NULL, robust = FALSE, model = "linear")
}

## The Bayesian quantile group LASSO model
data(data)
data = data$data_group
g=data$g
y=data$y
e=data$e

fit1=pqrBayes(g,y,e,d=3,model="group")
\donttest{

## Non-sparse robust Bayesian version
fit2 <- pqrBayes(g, y, e, d = 3, prior = "Laplace", model = "group")

## Non-robust Bayesian version
fit3 <- pqrBayes(g, y, e, d = 3, quant = NULL, robust = FALSE, model = "group")
}
## The regularized Bayesian quantile varying coefficient model
data(data)
data = data$data_varying
g=data$g
y=data$y
e=data$e
u=data$u
fit1=pqrBayes(g,y,u,e,model="VC")

\donttest{

## Non-sparse robust Bayesian example
fit2 <- pqrBayes(g, y, u, e, prior = "Laplace", model = "VC")

## Non-robust Bayesian example
fit3 <- pqrBayes(g, y, u, e, quant = NULL,robust = FALSE, model = "VC")
}
}
\references{
Fan, K., Subedi, S., Yang, G., Lu, X., Ren, J. and Wu, C. (2024). Is Seeing Believing? A Practitioner's Perspective on High-dimensional Statistical Inference in Cancer Genomics Studies. {\emph{Entropy}, 26(9).794} \doi{10.3390/e26090794}

Zhou, F., Ren, J., Ma, S. and Wu, C. (2023). The Bayesian regularized quantile varying coefficient model.
 {\emph{Computational Statistics & Data Analysis}, 187, 107808} \doi{10.1016/j.csda.2023.107808}
 
Ren, J., Zhou, F., Li, X., Ma, S., Jiang, Y. and Wu, C. (2023). Robust Bayesian variable selection for gene-environment interactions. 
{\emph{Biometrics}, 79(2), 684-694} \doi{10.1111/biom.13670}

Fan, K. and Wu, C. (2025). A New Robust Binary Bayesian LASSO. 
{\emph{Stat}, 14 (3), e70078} \doi{10.1002/sta4.70078}

Fan, K., Srijana, S., Dissanayake, V. and Wu, C. (2026). Robust Bayesian high-dimensional variable selection and inference with the horseshoe family of priors. 
{\emph{Computational Statistics & Data Analysis}, 219, 108358} \doi{10.1016/j.csda.2026.108358}

Ren, J., Zhou, F., Li, X., Chen, Q., Zhang, H., Ma, S., Jiang, Y. and Wu, C. (2020) Semi-parametric Bayesian variable selection for gene-environment interactions.
{\emph{Statistics in Medicine}, 39: 617– 638} \doi{10.1002/sim.8434}
}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data}
\alias{data}
\title{Simulated data under high-dimensional linear, binary, group LASSO and quantile varying coefficient models}
\format{
The data_linear object consists of 4 components: g, y, e and coeff. coeff contains the true values of parameters used for generating the response variable \eqn{y}.
The data_binary object consists of 4 components: g, y, e and coeff. coeff contains the true values of parameters used for generating the response variable \eqn{y}.
The data_group object consists of 4 components: g, y, e and coeff. coeff contains the true values of parameters used for generating the response variable \eqn{y}.
The data_varying object consists of five components: g, y, u, e and coeff. coeff contains the true values of parameters used for generating the response variable \eqn{y}.
}
\description{
Simulated data under high-dimensional linear, binary, group LASSO and quantile varying coefficient models
}
\details{
\strong{Generating Y using a sparse linear (quantile) regression model}

The true data generating model under sparse linear regression:
\deqn{Y_i=\beta_0+\beta_{1}X_{i1}+\beta_{2}X_{i2}+\beta_{3}X_{i3}+\epsilon_i,}
where \eqn{\epsilon_i\sim N(0,1)}, \eqn{\beta_{0}=0}, \eqn{\beta_{1}=1 }, \eqn{\beta_{2}=1.5} and \eqn{\beta_3=2}.

\strong{Generating Y using a sparse binary (quantile) regression model}

The true data generating model under sparse linear regression:
\deqn{\tilde{Y}_i=\beta_0+\beta_{1}X_{i1}+\beta_{2}X_{i2}+\beta_{3}X_{i3}+\epsilon_i,}
where \eqn{\epsilon_i\sim N(0,1)}, \eqn{\beta_{0}=0}, \eqn{\beta_{1}=0.22 }, \eqn{\beta_{2}=0.18} and \eqn{\beta_3=0.14}.

\eqn{Y_i=1} if \eqn{\tilde{Y}_i>0} and \eqn{Y_i=0} otherwise.

\strong{Generating Y using a high-dimensional group LASSO model}

The true data generating model under a group LASSO model:
\deqn{Y_i=\beta_0+\beta_{1}X_{i1}+\beta_{2}X_{i2}+\beta_{3}X_{i3}+\beta_{7}X_{i7}+\beta_{8}X_{i8}+\beta_{9}X_{i9}+\epsilon_i,}
where \eqn{\epsilon_i\sim N(0,1)}, \eqn{\beta_{0}=0}, \eqn{\beta_{1}=0.6}, \eqn{\beta_{2}=0.7},\eqn{\beta_{3}=0.8},\eqn{\beta_{7}=0.65}, \eqn{\beta_{8}=0.75} and \eqn{\beta_{9}=0.85}.

\strong{Generating Y using a (quantile) varying coefficient model}

Data generation under sparse (quantile) VC model:
\deqn{Y_i=\gamma_0(v_i)+\gamma_1(v_i)X_{i1}+\gamma_2(v_i)X_{i2}+\gamma_3(v_i)X_{i3}+\epsilon_i,}
where \eqn{\epsilon_i\sim N(0,1)}, \eqn{\gamma_{0}(v_i)=1.5\sin(0.2\pi*v_i}), \eqn{\gamma_{1}(v_i)=2\exp(0.2v_i-1)-1.5 }, \eqn{\gamma_{2}(v_i)=2-2v_i } and \eqn{\gamma_3(v_i)=-4+(v_i-2)^3/6}.
}
\examples{
data(data)
data = data$data_linear
g=data$g
dim(g)
y=data$y
coeff=data$coeff
print(coeff)

data = data$data_binary
g=data$g
dim(g)
y=data$y
coeff=data$coeff
print(coeff)

data = data$data_group
g=data$g
dim(g)
y=data$y
coeff=data$coeff
print(coeff)

data = data$data_varying
g=data$g
dim(g)
coeff=data$coeff
print(coeff)


}
\seealso{
\code{\link{pqrBayes}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.pqrBayes.R
\name{print.pqrBayes}
\alias{print.pqrBayes}
\title{print a pqrBayes result}
\usage{
\method{print}{pqrBayes}(x, digits = max(3, getOption("digits") - 3), \dots)
}
\arguments{
\item{x}{pqrBayes result.}

\item{digits}{significant digits in printout.}

\item{...}{other print arguments.}
}
\value{
No return value, called for side effects.
}
\description{
Print a pqrBayes result
}
\seealso{
\code{\link{pqrBayes}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.pqrBayes.R
\name{predict_pqrBayes}
\alias{predict_pqrBayes}
\title{Make predictions from a pqrBayes object}
\usage{
predict_pqrBayes(object, g.new, u.new, e.new, y.new, robust, quant, model, ...)
}
\arguments{
\item{object}{a pqrBayes object.}

\item{g.new}{a matrix of new predictors (e.g. genetic factors) at which predictions are to be made. When being applied to the sparse linear, binary LASSO or group LASSO, g.new = g.}

\item{u.new}{a vector of new environmental factor at which predictions are to be made. When being applied to the sparse linear model, binary LASSO or group LASSO, u.new = NULL. The default value is NULL.}

\item{e.new}{a vector or matrix of new clinical covariates at which predictions are to be made. The default value is NULL.}

\item{y.new}{a vector of the response of new observations. When being applied to the sparse linear model, binary LASSO or group LASSO, y.new = y.}

\item{robust}{logical flag. If TRUE, robust methods are used. Otherwise, non-robust methods are used. The default value is TRUE.}

\item{quant}{the quantile level specified by users. Required when robust = TRUE. Ignored (set to NULL) when robust = FALSE.The default value is 0.5.}

\item{model}{the model to be fitted. The default is "VC" for a quantile varying coefficient model. Users can also specify "linear" for a sparse linear model, "binary" for binary LASSO and "group" for a group LASSO.}

\item{...}{other predict arguments}
}
\value{
an object of class `pqrBayes.pred' is returned, which is a list with components:
\item{error}{prediction error.}
\item{y.pred}{predicted values of the new observations.}
}
\description{
Make predictions from a pqrBayes object
}
\details{
g.new (u.new) must have the same number of columns as g (u) used for fitting the model. By default, the clinical covariates are NULL unless 
provided. The predictions are made based on the posterior estimates of coefficients in the pqrBayes object.
}
\examples{
## The quantile regression model
data(data)
data = data$data_linear
g=data$g
y=data$y
e=data$e
fit1=pqrBayes(g,y,e,model="linear")
prediction=predict_pqrBayes(fit1,g, y.new = y, model="linear")
}
\seealso{
\code{\link{pqrBayes}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimation.R
\name{estimation.pqrBayes}
\alias{estimation.pqrBayes}
\title{Estimation and estimation accuracy for a pqrBayes object}
\usage{
estimation.pqrBayes(object,coefficient,u.grid=NULL,model="linear")
}
\arguments{
\item{object}{an object of class `pqrBayes'.}

\item{coefficient}{the vector of quantile regression coefficients under a sparse linear model (with a continuous response), binary LASSO and group LASSO or the matrix of true varying coefficients evaluated on the grid points under a varying coefficient model.}

\item{u.grid}{the vector of grid points under a varying coefficient model. When fitting a sparse linear model, binary LASSO or group LASSO, u.grid = NULL.}

\item{model}{the model to be fitted. Users can choose "linear" for a sparse linear model (with a continuous response), "binary" for binary LASSO, "group" for group LASSO or "VC" for a varying coefficient model.}
}
\value{
an object of class `pqrBayes.est' is returned, which is a list with components:
\item{error}{mean square error or integrated mean square errors and total integrated mean square error.}
\item{coeff.est}{estimated values of the regression coefficients or the varying coefficients.}
}
\description{
Calculate estimated regression coefficients with estimation accuracy from a sparse linear model (with a continuous response), binary LASSO, group LASSO and quantile VC models, respectively.
}
\examples{
## The quantile regression model
data(data)
data = data$data_linear
g=data$g
y=data$y
e=data$e
coeff = data$coeff
fit1=pqrBayes(g,y,e,d = NULL,quant=0.5,model="linear")
estimation=estimation.pqrBayes(fit1,coeff,model="linear")

}
\seealso{
\code{\link{pqrBayes}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.pqrBayes.R
\name{print.pqrBayes.pred}
\alias{print.pqrBayes.pred}
\title{print a pqrBayes.pred object}
\usage{
\method{print}{pqrBayes.pred}(x, digits = max(3, getOption("digits") - 3), \dots)
}
\arguments{
\item{x}{pqrBayes.pred object.}

\item{digits}{significant digits in printout.}

\item{...}{other print arguments}
}
\value{
No return value, called for side effects.
}
\description{
Print a summary of a pqrBayes.pred object
}
\seealso{
\code{\link{predict_pqrBayes}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/select.pqrBayes.R
\name{pqrBayes.select}
\alias{pqrBayes.select}
\title{Variable selection for a pqrBayes object}
\usage{
pqrBayes.select(object,prior="SS",model="linear")
}
\arguments{
\item{object}{a pqrBayes object.}

\item{prior}{the prior used in the pqrBayes function. Users can choose "SS" for the spike-and-slab prior, "HS" for the horseshoe prior, "HS+" for the horseshoe plus prior, "RHS" for the regularized horseshoe prior and "Laplace" for the Laplace prior. The default value is "SS".}

\item{model}{the model to be fitted. Users can also choose "linear" for a sparse linear model (with a continuous response), "VC" for a varying coefficient model or "group" for group LASSO.}
}
\value{
an object of class `select' is returned, which includes the indices of the selected predictors (e.g. genetic factors).
}
\description{
Variable selection for a pqrBayes object
}
\details{
For class `Sparse', the median probability model (MPM) (Barbieri and Berger, 2004) is used to identify predictors that are significantly associated
with the response variable. For class `NonSparse', variable selection is based on 95\% credible interval.
Please check the references for more details about the variable selection.
}
\examples{
## The sparse quantile regression model
data(data)
data = data$data_linear
g=data$g
y=data$y
e=data$e

fit1=pqrBayes(g,y,e,d = NULL,quant=0.5,model="linear")
select=pqrBayes.select(obj = fit1,prior = "SS",model="linear")

## The quantile varying coefficient model
data(data)
data = data$data_varying
g=data$g
y=data$y
e=data$e
fit1=pqrBayes(g,y,e,d = NULL,quant=0.5,model="VC")
select=pqrBayes.select(obj = fit1,prior = "SS",model="VC")
select

\donttest{
## Non-sparse example with VC model
fit2 <- pqrBayes(
  g = g, y = y, e = e, d = NULL,
  quant = 0.5,
  prior= "Laplace",
  model = "VC"
)

select <- pqrBayes.select(obj = fit2, prior = "SS", model = "VC")
select
}

}
\references{
Ren, J., Zhou, F., Li, X., Ma, S., Jiang, Y. and Wu, C. (2023). Robust Bayesian variable selection for gene-environment interactions. 
{\emph{Biometrics}, 79(2), 684-694} \doi{10.1111/biom.13670}

Barbieri, M.M. and Berger, J.O. (2004). Optimal predictive model selection. {\emph{Ann. Statist}, 32(3):870–897}
}
\seealso{
\code{\link{pqrBayes}}
}
